@@ -1,30 +1,72 @@
-require "vagrant/shared_helpers"
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
 
-require 'rubygems'
-require 'log4r'
+require "log4r"
+
+# Add patches to log4r to support trace level
+require "vagrant/patches/log4r"
+# Set our log levels and include trace
+require 'log4r/configurator'
+Log4r::Configurator.custom_levels(*(["TRACE"] + Log4r::Log4rConfig::LogLevels))
+
+# Update the default formatter within the log4r library to ensure
+# sensitive values are being properly scrubbed from logger data
+class Log4r::BasicFormatter
+  alias_method :vagrant_format_object, :format_object
+  def format_object(obj)
+    Vagrant::Util::CredentialScrubber.desensitize(vagrant_format_object(obj))
+  end
+end
+
+require "optparse"
+
+module Vagrant
+  # This is a customized OptionParser for Vagrant plugins. It
+  # will automatically add any default CLI options defined
+  # outside of command implementations to the local option
+  # parser instances in use
+  class OptionParser < ::OptionParser
+    def initialize(*_)
+      super
+      Vagrant.default_cli_options.each do |opt_proc|
+        opt_proc.call(self)
+      end
+    end
+  end
+end
+
+# Inject the option parser into the vagrant plugins
+# module so it is automatically used when defining
+# command options
+module VagrantPlugins
+  OptionParser = Vagrant::OptionParser
+end
+
+# Load in our helpers and utilities
+require "vagrant/shared_helpers"
+require "rubygems"
+require "vagrant/util"
+require "vagrant/plugin/manager"
 
 # Enable logging if it is requested. We do this before
 # anything else so that we can setup the output before
 # any logging occurs.
-if ENV["VAGRANT_LOG"] && ENV["VAGRANT_LOG"] != ""
-  # Require Log4r and define the levels we'll be using
-  require 'log4r/config'
-  Log4r.define_levels(*Log4r::Log4rConfig::LogLevels)
 
-  level = nil
-  begin
-    level = Log4r.const_get(ENV["VAGRANT_LOG"].upcase)
-  rescue NameError
-    # This means that the logging constant wasn't found,
-    # which is fine. We just keep `level` as `nil`. But
-    # we tell the user.
-    level = nil
+# NOTE: We must do this little hack to allow
+# rest-client to write using the `<<` operator.
+# See https://github.com/rest-client/rest-client/issues/34#issuecomment-290858
+# for more information
+class VagrantLogger < Log4r::Logger
+  def << msg
+    debug(msg.strip)
   end
+end
 
-  # Some constants, such as "true" resolve to booleans, so the
-  # above error checking doesn't catch it. This will check to make
-  # sure that the log level is an integer, as Log4r requires.
-  level = nil if !level.is_a?(Integer)
+if ENV["VAGRANT_LOG"] && ENV["VAGRANT_LOG"] != ""
+  level = Log4r::LNAMES.index(ENV["VAGRANT_LOG"].upcase)
+  if level.nil?
+    level = Log4r::LNAMES.index("FATAL")
+  end
 
   if !level
     # We directly write to stderr here because the VagrantError system
@@ -38,10 +80,26 @@ if ENV["VAGRANT_LOG"] && ENV["VAGRANT_LOG"] != ""
   # Set the logging level on all "vagrant" namespaced
   # logs as long as we have a valid level.
   if level
-    logger = Log4r::Logger.new("vagrant")
-    logger.outputters = Log4r::Outputter.stderr
-    logger.level = level
-    logger = nil
+    ["vagrant", "vagrantplugins"].each do |lname|
+      logger = VagrantLogger.new(lname)
+      if ENV["VAGRANT_LOG_FILE"] && ENV["VAGRANT_LOG_FILE"] != ""
+        logger.outputters = Log4r::FileOutputter.new("vagrant", filename: ENV["VAGRANT_LOG_FILE"])
+      else
+        logger.outputters = Log4r::Outputter.stderr
+      end
+      logger.level = level
+    end
+    Log4r::RootLogger.instance.level = level
+
+    base_formatter = Log4r::BasicFormatter.new
+    if ENV["VAGRANT_LOG_TIMESTAMP"]
+      base_formatter = Log4r::PatternFormatter.new(
+        pattern: "%d [%5l] %m",
+        date_pattern: "%F %T"
+      )
+    end
+
+    Log4r::Outputter.stderr.formatter = Vagrant::Util::LoggingFormatter.new(base_formatter)
   end
 end
 
@@ -59,11 +117,31 @@ require 'openssl'
 # Always make the version available
 require 'vagrant/version'
 global_logger = Log4r::Logger.new("vagrant::global")
+Vagrant.global_logger = global_logger
 global_logger.info("Vagrant version: #{Vagrant::VERSION}")
 global_logger.info("Ruby version: #{RUBY_VERSION}")
 global_logger.info("RubyGems version: #{Gem::VERSION}")
 ENV.each do |k, v|
-  global_logger.info("#{k}=#{v.inspect}") if k =~ /^VAGRANT_/
+  next if k.start_with?("VAGRANT_OLD")
+  global_logger.info("#{k}=#{v.inspect}") if k.start_with?("VAGRANT_")
+end
+
+# If the vagrant_ssl library exists, a recent version
+# of openssl is in use and its needed to load all the
+# providers needed
+if File.exist?(File.expand_path("vagrant/vagrant_ssl.so", __dir__))
+  global_logger.debug("vagrant ssl helper found for loading ssl providers")
+  begin
+    require "vagrant/vagrant_ssl"
+    Vagrant.vagrant_ssl_load
+    global_logger.debug("ssl providers successfully loaded")
+  rescue LoadError => err
+    global_logger.warn("failed to load ssl providers, attempting to continue (#{err})")
+  rescue => err
+    global_logger.warn("unexpected failure loading ssl providers, attempting to continue (#{err})")
+  end
+else
+  global_logger.warn("vagrant ssl helper was not found, continuing...")
 end
 
 # We need these components always so instead of an autoload we
@@ -72,24 +150,29 @@ require "vagrant/plugin"
 require "vagrant/registry"
 
 module Vagrant
-  autoload :Action,        'vagrant/action'
-  autoload :BatchAction,   'vagrant/batch_action'
-  autoload :Box,           'vagrant/box'
-  autoload :BoxCollection, 'vagrant/box_collection'
-  autoload :CLI,           'vagrant/cli'
-  autoload :Command,       'vagrant/command'
-  autoload :Config,        'vagrant/config'
-  autoload :Driver,        'vagrant/driver'
-  autoload :Environment,   'vagrant/environment'
-  autoload :Errors,        'vagrant/errors'
-  autoload :Guest,         'vagrant/guest'
-  autoload :Host,          'vagrant/host'
-  autoload :Machine,       'vagrant/machine'
-  autoload :MachineIndex,  'vagrant/machine_index'
-  autoload :MachineState,  'vagrant/machine_state'
-  autoload :Plugin,        'vagrant/plugin'
-  autoload :UI,            'vagrant/ui'
-  autoload :Util,          'vagrant/util'
+  autoload :Action,         'vagrant/action'
+  autoload :Alias,          'vagrant/alias'
+  autoload :BatchAction,    'vagrant/batch_action'
+  autoload :Box,            'vagrant/box'
+  autoload :BoxCollection,  'vagrant/box_collection'
+  autoload :BoxMetadata,    'vagrant/box_metadata'
+  autoload :Bundler,        'vagrant/bundler'
+  autoload :CLI,            'vagrant/cli'
+  autoload :CapabilityHost, 'vagrant/capability_host'
+  autoload :Config,         'vagrant/config'
+  autoload :Environment,    'vagrant/environment'
+  autoload :Errors,         'vagrant/errors'
+  autoload :Guest,          'vagrant/guest'
+  autoload :Host,           'vagrant/host'
+  autoload :Machine,        'vagrant/machine'
+  autoload :MachineIndex,   'vagrant/machine_index'
+  autoload :MachineState,   'vagrant/machine_state'
+  autoload :Plugin,         'vagrant/plugin'
+  autoload :Registry,       'vagrant/registry'
+  autoload :UI,             'vagrant/ui'
+  autoload :Util,           'vagrant/util'
+  autoload :Vagrantfile,    'vagrant/vagrantfile'
+  autoload :VERSION,        'vagrant/version'
 
   # These are the various plugin versions and their components in
   # a lazy loaded Hash-like structure.
@@ -113,6 +196,8 @@ module Vagrant
     c.register([:"2", :provisioner])  { Plugin::V2::Provisioner }
     c.register([:"2", :push])         { Plugin::V2::Push }
     c.register([:"2", :synced_folder]) { Plugin::V2::SyncedFolder }
+
+    c.register(:remote)               { Plugin::Remote::Plugin }
   end
 
   # Configure a Vagrant environment. The version specifies the version
@@ -138,16 +223,9 @@ module Vagrant
       return true if plugin("2").manager.registered.any? { |p| p.name == name }
     end
 
-    # Make the requirement object
-    version = Gem::Requirement.new([version]) if version
-
     # Now check the plugin gem names
     require "vagrant/plugin/manager"
-    Plugin::Manager.instance.installed_specs.any? do |s|
-      match = s.name == name
-      next match if !version
-      next match && version.satisfied_by?(s.version)
-    end
+    Plugin::Manager.instance.plugin_installed?(name, version)
   end
 
   # Returns a superclass to use when creating a plugin for Vagrant.
@@ -189,6 +267,17 @@ module Vagrant
     puts "be removed in the next version of Vagrant."
   end
 
+  # This checks if Vagrant is installed in a specific version.
+  #
+  # Example:
+  #
+  #    Vagrant.version?(">= 2.1.0")
+  #
+  def self.version?(*requirements)
+    req = Gem::Requirement.new(*requirements)
+    req.satisfied_by?(Gem::Version.new(VERSION))
+  end
+
   # This allows a Vagrantfile to specify the version of Vagrant that is
   # required. You can specify a list of requirements which will all be checked
   # against the running Vagrant version.
@@ -205,8 +294,7 @@ module Vagrant
     logger = Log4r::Logger.new("vagrant::root")
     logger.info("Version requirements from Vagrantfile: #{requirements.inspect}")
 
-    req = Gem::Requirement.new(*requirements)
-    if req.satisfied_by?(Gem::Version.new(VERSION))
+    if version?(*requirements)
       logger.info("  - Version requirements satisfied!")
       return
     end
@@ -226,7 +314,9 @@ module Vagrant
       ENV.each do |k,v|
         if k.start_with?("VAGRANT_OLD_ENV")
           key = k.sub(/^VAGRANT_OLD_ENV_/, "")
-          h[key] = v
+          if !key.empty?
+            h[key] = v
+          end
         end
       end
     end
@@ -242,33 +332,10 @@ if I18n.config.respond_to?(:enforce_available_locales=)
   I18n.config.enforce_available_locales = true
 end
 
-# Setup the plugin manager and load any defined plugins
-require_relative "vagrant/plugin/manager"
-plugins = Vagrant::Plugin::Manager.instance.installed_plugins
-
-global_logger.info("Plugins:")
-plugins.each do |plugin_name, plugin_info|
-  installed_version = plugin_info["installed_gem_version"]
-  version_constraint = plugin_info["gem_version"]
-  installed_version = 'undefined' if installed_version.to_s.empty?
-  version_constraint = '> 0' if version_constraint.to_s.empty?
-  global_logger.info(
-    "  - #{plugin_name} = [installed: " \
-      "#{installed_version} constraint: " \
-      "#{version_constraint}]"
-  )
-end
-
-if Vagrant.plugins_init?
-  begin
-    Vagrant::Bundler.instance.init!(plugins)
-  rescue Exception => e
-    global_logger.error("Plugin initialization error - #{e.class}: #{e}")
-    e.backtrace.each do |backtrace_line|
-      global_logger.debug(backtrace_line)
-    end
-    raise Vagrant::Errors::PluginInitError, message: e.to_s
-  end
+if Vagrant.enable_resolv_replace
+  global_logger.info("resolv replacement has been enabled!")
+else
+  global_logger.warn("resolv replacement has not been enabled!")
 end
 
 # A lambda that knows how to load plugins from a single directory.
@@ -300,45 +367,3 @@ Vagrant.source_root.join("plugins").children(true).each do |directory|
   # Otherwise, attempt to load from sub-directories
   directory.children(true).each(&plugin_load_proc)
 end
-
-# If we have plugins enabled, then load those
-if Vagrant.plugins_enabled?
-  begin
-    global_logger.info("Loading plugins!")
-    plugins.each do |plugin_name, plugin_info|
-      if plugin_info["require"].to_s.empty?
-        begin
-          global_logger.debug("Loading plugin `#{plugin_name}` with default require: `#{plugin_name}`")
-          require plugin_name
-        rescue LoadError, Gem::LoadError => load_error
-          if plugin_name.include?("-")
-            begin
-              plugin_slash = plugin_name.gsub("-", "/")
-              global_logger.debug("Failed to load plugin `#{plugin_name}` with default require.")
-              global_logger.debug("Loading plugin `#{plugin_name}` with slash require: `#{plugin_slash}`")
-              require plugin_slash
-            rescue LoadError, Gem::LoadError
-              raise load_error
-            end
-          end
-        end
-      else
-        global_logger.debug("Loading plugin `#{plugin_name}` with custom require: `#{plugin_info["require"]}`")
-        require plugin_info["require"]
-      end
-      global_logger.debug("Successfully loaded plugin `#{plugin_name}`.")
-    end
-    if defined?(::Bundler)
-      global_logger.debug("Bundler detected in use. Loading `:plugins` group.")
-      ::Bundler.require(:plugins)
-    end
-  rescue Exception => e
-    global_logger.error("Plugin loading error: #{e.class} - #{e}")
-    e.backtrace.each do |backtrace_line|
-      global_logger.debug(backtrace_line)
-    end
-    raise Vagrant::Errors::PluginLoadError, message: e.to_s
-  end
-else
-  global_logger.debug("Plugin loading is currently disabled.")
-end
