@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "digest/sha1"
 require "log4r"
 require "pathname"
@@ -6,6 +9,7 @@ require "uri"
 require "vagrant/box_metadata"
 require "vagrant/util/downloader"
 require "vagrant/util/file_checksum"
+require "vagrant/util/file_mutex"
 require "vagrant/util/platform"
 
 module Vagrant
@@ -25,6 +29,7 @@ module Vagrant
         def initialize(app, env)
           @app    = app
           @logger = Log4r::Logger.new("vagrant::action::builtin::box_add")
+          @parser = URI::RFC2396_Parser.new
         end
 
         def call(env)
@@ -44,7 +49,7 @@ module Vagrant
             u = u.gsub("\\", "/")
             if Util::Platform.windows? && u =~ /^[a-z]:/i
               # On Windows, we need to be careful about drive letters
-              u = "file:///#{URI.escape(u)}"
+              u = "file:///#{@parser.escape(u)}"
             end
 
             if u =~ /^[a-z0-9]+:.*$/i && !u.start_with?("file://")
@@ -53,9 +58,9 @@ module Vagrant
             end
 
             # Expand the path and try to use that, if possible
-            p = File.expand_path(URI.unescape(u.gsub(/^file:\/\//, "")))
+            p = File.expand_path(@parser.unescape(u.gsub(/^file:\/\//, "")))
             p = Util::Platform.cygwin_windows_path(p)
-            next "file://#{URI.escape(p.gsub("\\", "/"))}" if File.file?(p)
+            next "file://#{@parser.escape(p.gsub("\\", "/"))}" if File.file?(p)
 
             u
           end
@@ -63,17 +68,29 @@ module Vagrant
           # If we received a shorthand URL ("mitchellh/precise64"),
           # then expand it properly.
           expanded = false
-          url.each_index do |i|
-            next if url[i] !~ /^[^\/]+\/[^\/]+$/
+          # Mark if only a single url entry was provided
+          single_entry = url.size == 1
 
-            if !File.file?(url[i])
+          url = url.map do |url_entry|
+            if url_entry =~ /^[^\/]+\/[^\/]+$/ && !File.file?(url_entry)
               server = Vagrant.server_url env[:box_server_url]
               raise Errors::BoxServerNotSet if !server
 
               expanded = true
-              url[i] = "#{server}/#{url[i]}"
+              # If only a single entry, expand to both the API endpoint and
+              # the direct shorthand endpoint.
+              if single_entry
+                url_entry = [
+                  "#{server}/api/v2/vagrant/#{url_entry}",
+                  "#{server}/#{url_entry}"
+                ]
+              else
+                url_entry = "#{server}/#{url_entry}"
+              end
             end
-          end
+
+            url_entry
+          end.flatten
 
           # Call the hook to transform URLs into authenticated URLs.
           # In the case we don't have a plugin that does this, then it
@@ -94,6 +111,21 @@ module Vagrant
             end
           end
 
+          # If only a single entry was provided, and it was expanded,
+          # inspect the metadata check results and extract the one that
+          # was successful, with preference to the API endpoint
+          if single_entry && expanded
+            idx = is_metadata_results.index { |v| v === true }
+            # If none of the urls were successful, set the index
+            # as the last entry
+            idx = is_metadata_results.size - 1 if idx.nil?
+
+            # Now reset collections with single value
+            is_metadata_results = [is_metadata_results[idx]]
+            authed_urls = [authed_urls[idx]]
+            url = [url[idx]]
+          end
+
           if expanded && url.length == 1
             is_error = is_metadata_results.find do |b|
               b.is_a?(Errors::DownloaderError)
@@ -107,6 +139,14 @@ module Vagrant
             end
           end
 
+          is_error = is_metadata_results.find do |b|
+            b.is_a?(Errors::DownloaderError)
+          end
+          if is_error
+            raise Errors::BoxMetadataDownloadError,  
+              message: is_error.extra_data[:message]
+          end
+
           is_metadata = is_metadata_results.any? { |b| b === true }
           if is_metadata && url.length > 1
             raise Errors::BoxAddMetadataMultiURL,
@@ -117,7 +157,7 @@ module Vagrant
             url = [url.first, authed_urls.first]
             add_from_metadata(url, env, expanded)
           else
-            add_direct(url, env)
+            add_direct(authed_urls, env)
           end
 
           @app.call(env)
@@ -152,6 +192,7 @@ module Vagrant
             env,
             checksum: env[:box_checksum],
             checksum_type: env[:box_checksum_type],
+            architecture: env[:architecture]
           )
         end
 
@@ -166,6 +207,9 @@ module Vagrant
         #   a Atlas server URL.
         def add_from_metadata(url, env, expanded)
           original_url = env[:box_url]
+          architecture = env[:box_architecture]
+          display_architecture = architecture == :auto ?
+                                   Util::Platform.architecture : architecture
           provider = env[:box_provider]
           provider = Array(provider) if provider
           version = env[:box_version]
@@ -196,7 +240,7 @@ module Vagrant
             return if @download_interrupted
 
             File.open(metadata_path) do |f|
-              metadata = BoxMetadata.new(f)
+              metadata = BoxMetadata.new(f, url: authenticated_url)
             end
           rescue Errors::DownloaderError => e
             raise if !expanded
@@ -215,12 +259,17 @@ module Vagrant
           end
 
           metadata_version  = metadata.version(
-            version || ">= 0", provider: provider)
+            version || ">= 0",
+            provider: provider,
+            architecture: architecture,
+          )
           if !metadata_version
-            if provider && !metadata.version(">= 0", provider: provider)
+            if provider && !metadata.version(">= 0", provider: provider, architecture: architecture)
               raise Errors::BoxAddNoMatchingProvider,
                 name: metadata.name,
-                requested: provider,
+                requested: [provider,
+                  display_architecture ? "(#{display_architecture})" : nil
+                ].compact.join(" "),
                 url: display_url
             else
               raise Errors::BoxAddNoMatchingVersion,
@@ -236,16 +285,16 @@ module Vagrant
             # If a provider was specified, make sure we get that specific
             # version.
             provider.each do |p|
-              metadata_provider = metadata_version.provider(p)
+              metadata_provider = metadata_version.provider(p, architecture)
               break if metadata_provider
             end
-          elsif metadata_version.providers.length == 1
+          elsif metadata_version.providers(architecture).length == 1
             # If we have only one provider in the metadata, just use that
             # provider.
             metadata_provider = metadata_version.provider(
-              metadata_version.providers.first)
+              metadata_version.providers.first, architecture)
           else
-            providers = metadata_version.providers.sort
+            providers = metadata_version.providers(architecture).sort
 
             choice = 0
             options = providers.map do |p|
@@ -266,7 +315,7 @@ module Vagrant
             end
 
             metadata_provider = metadata_version.provider(
-              providers[choice-1])
+              providers[choice-1], architecture)
           end
 
           provider_url = metadata_provider.url
@@ -280,6 +329,17 @@ module Vagrant
             provider_url = authed_urls[0]
           end
 
+          # The architecture name used when adding the box should be
+          # the value extracted from the metadata provider
+          arch_name = metadata_provider.architecture
+
+          # In the special case where the architecture name is "unknown" and
+          # it is listed as the default architecture, unset the architecture
+          # name so it is installed without architecture information
+          if arch_name == "unknown" && metadata_provider.default_architecture
+            arch_name = nil
+          end
+
           box_add(
             [[provider_url, metadata_provider.url]],
             metadata.name,
@@ -289,6 +349,7 @@ module Vagrant
             env,
             checksum: metadata_provider.checksum,
             checksum_type: metadata_provider.checksum_type,
+            architecture: arch_name,
           )
         end
 
@@ -304,16 +365,21 @@ module Vagrant
         # @param [Hash] env
         # @return [Box]
         def box_add(urls, name, version, provider, md_url, env, **opts)
+          display_architecture = opts[:architecture] == :auto ?
+                                   Util::Platform.architecture : opts[:architecture]
           env[:ui].output(I18n.t(
             "vagrant.box_add_with_version",
             name: name,
             version: version,
-            providers: Array(provider).join(", ")))
+            providers: [
+              provider,
+              display_architecture ? "(#{display_architecture})" : nil
+            ].compact.join(" ")))
 
           # Verify the box we're adding doesn't already exist
           if provider && !env[:box_force]
             box = env[:box_collection].find(
-              name, provider, version)
+              name, provider, version, opts[:architecture])
             if box
               raise Errors::BoxAlreadyExists,
                 name: name,
@@ -348,9 +414,15 @@ module Vagrant
             end
 
             if opts[:checksum] && opts[:checksum_type]
-              env[:ui].detail(I18n.t("vagrant.actions.box.add.checksumming"))
-              validate_checksum(
-                opts[:checksum_type], opts[:checksum], box_url)
+              if opts[:checksum].to_s.strip.empty?
+                @logger.warn("Given checksum is empty, cannot validate checksum for box")
+              elsif opts[:checksum_type].to_s.strip.empty?
+                @logger.warn("Given checksum type is empty, cannot validate checksum for box")
+              else
+                env[:ui].detail(I18n.t("vagrant.actions.box.add.checksumming"))
+                validate_checksum(
+                  opts[:checksum_type], opts[:checksum], box_url)
+              end
             end
 
             # Add the box!
@@ -358,7 +430,9 @@ module Vagrant
               box_url, name, version,
               force: env[:box_force],
               metadata_url: md_url,
-              providers: provider)
+              providers: provider,
+              architecture: opts[:architecture]
+            )
           ensure
             # Make sure we delete the temporary file after we add it,
             # unless we were interrupted, in which case we keep it around
@@ -377,7 +451,10 @@ module Vagrant
             "vagrant.box_added",
             name: box.name,
             version: box.version,
-            provider: box.provider))
+            provider: [
+              provider,
+              display_architecture ? "(#{display_architecture})" : nil
+            ].compact.join(" ")))
 
           # Store the added box in the env for future middleware
           env[:box_added] = box
@@ -427,14 +504,19 @@ module Vagrant
           downloader_options[:headers] = ["Accept: application/json"] if opts[:json]
           downloader_options[:ui] = env[:ui] if opts[:ui]
           downloader_options[:location_trusted] = env[:box_download_location_trusted]
+          downloader_options[:disable_ssl_revoke_best_effort] = env[:box_download_disable_ssl_revoke_best_effort]
+          downloader_options[:box_extra_download_options] = env[:box_extra_download_options]
 
-          Util::Downloader.new(url, temp_path, downloader_options)
+          d = Util::Downloader.new(url, temp_path, downloader_options)
+          env[:hook].call(:authenticate_box_downloader, downloader: d)
+          d
         end
 
         def download(url, env, **opts)
           opts[:ui] = true if !opts.key?(:ui)
 
           d = downloader(url, env, **opts)
+          env[:hook].call(:authenticate_box_downloader, downloader: d)
 
           # Download the box to a temporary path. We store the temporary
           # path as an instance variable so that the `#recover` method can
@@ -460,12 +542,21 @@ module Vagrant
           end
 
           begin
-            d.download!
-          rescue Errors::DownloaderInterrupted
-            # The downloader was interrupted, so just return, because that
-            # means we were interrupted as well.
-            @download_interrupted = true
-            env[:ui].info(I18n.t("vagrant.actions.box.download.interrupted"))
+            mutex_path = d.destination + ".lock"
+            Util::FileMutex.new(mutex_path).with_lock do
+              begin
+                d.download!
+              rescue Errors::DownloaderInterrupted
+                # The downloader was interrupted, so just return, because that
+                # means we were interrupted as well.
+                @download_interrupted = true
+                env[:ui].info(I18n.t("vagrant.actions.box.download.interrupted"))
+              end
+            end
+          rescue Errors::VagrantLocked
+            raise Errors::DownloadAlreadyInProgress,
+              dest_path: d.destination,
+              lock_file_path: mutex_path
           end
 
           Pathname.new(d.destination)
@@ -478,6 +569,7 @@ module Vagrant
         # @return [Boolean] true if metadata
         def metadata_url?(url, env)
           d = downloader(url, env, json: true, ui: false)
+          env[:hook].call(:authenticate_box_downloader, downloader: d)
 
           # If we're downloading a file, cURL just returns no
           # content-type (makes sense), so we just test if it is JSON
@@ -488,7 +580,7 @@ module Vagrant
             url ||= uri.opaque
             #7570 Strip leading slash left in front of drive letter by uri.path
             Util::Platform.windows? && url.gsub!(/^\/([a-zA-Z]:)/, '\1')
-            url = URI.unescape(url)
+            url = @parser.unescape(url)
 
             begin
               File.open(url, "r") do |f|
@@ -498,7 +590,7 @@ module Vagrant
                   return false
                 end
 
-                BoxMetadata.new(f)
+                BoxMetadata.new(f, url: url)
               end
               return true
             rescue Errors::BoxMetadataMalformed
@@ -526,23 +618,14 @@ module Vagrant
           !!(match.last.chomp =~ /application\/json/)
         end
 
-        def validate_checksum(checksum_type, checksum, path)
-          checksum_klass = case checksum_type.to_sym
-          when :md5
-            Digest::MD5
-          when :sha1
-            Digest::SHA1
-          when :sha256
-            Digest::SHA2
-          else
-            raise Errors::BoxChecksumInvalidType,
-              type: checksum_type.to_s
-          end
-
-          @logger.info("Validating checksum with #{checksum_klass}")
+        def validate_checksum(checksum_type, _checksum, path)
+          checksum = _checksum.strip()
+          @logger.info("Validating checksum with #{checksum_type}")
           @logger.info("Expected checksum: #{checksum}")
 
-          actual = FileChecksum.new(path, checksum_klass).checksum
+          _actual = FileChecksum.new(path, checksum_type).checksum
+          actual = _actual.strip()
+          @logger.info("Actual checksum: #{actual}")
           if actual.casecmp(checksum) != 0
             raise Errors::BoxChecksumMismatch,
               actual: actual,
