@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require 'fileutils'
 require "pathname"
 
@@ -13,6 +16,7 @@ module Vagrant
       #
       #   * package.output - The filename of the outputted package.
       #   * package.include - An array of files to include in the package.
+      #   * package.info - Path of desired info.json file to include
       #   * package.directory - The directory which contains the contents to
       #       compress into the package.
       #
@@ -63,6 +67,7 @@ module Vagrant
           @app = app
 
           env["package.files"]  ||= {}
+          env["package.info"]   ||= ""
           env["package.output"] ||= "package.box"
 
           @fullpath = self.class.fullpath(env["package.output"])
@@ -77,11 +82,16 @@ module Vagrant
 
           raise Errors::PackageOutputDirectory if File.directory?(fullpath)
 
+          raise Errors::PackageInvalidInfo if invalid_info?
+
           @app.call(env)
 
           @env[:ui].info I18n.t("vagrant.actions.general.package.compressing", fullpath: fullpath)
+
           copy_include_files
+          copy_info
           setup_private_key
+          write_metadata_json
           compress
         end
 
@@ -121,7 +131,7 @@ module Vagrant
             @env[:ui].info I18n.t("vagrant.actions.general.package.packaging", file: from)
             FileUtils.mkdir_p(to.parent)
 
-            # Copy direcotry contents recursively.
+            # Copy directory contents recursively.
             if File.directory?(from)
               FileUtils.cp_r(Dir.glob(from), to.parent, preserve: true)
             else
@@ -135,6 +145,16 @@ module Vagrant
           raise Errors::PackageIncludeSymlink
         end
 
+        # This method copies the specified info.json file to the temporary directory
+        # so that it is accessible via the 'box list -i' command
+        def copy_info
+          info_path = Pathname.new(@env["package.info"])
+
+          if info_path.file?
+            FileUtils.cp(info_path, @env["package.directory"], preserve: true)
+          end
+        end
+
         # Compress the exported file into a package
         def compress
           # Get the output path. We have to do this up here so that the
@@ -151,6 +171,22 @@ module Vagrant
           end
         end
 
+        # Write the metadata file into the box so that the provider
+        # can be automatically detected when adding the box
+        def write_metadata_json
+          meta_path = File.join(@env["package.directory"], "metadata.json")
+          return if File.exist?(meta_path)
+
+          if @env[:machine] && @env[:machine].provider_name
+            provider_name = @env[:machine].provider_name
+          elsif @env[:env] && @env[:env].default_provider
+            provider_name = @env[:env].default_provider
+          else
+            return
+          end
+          File.write(meta_path, {provider: provider_name}.to_json)
+        end
+
         # This will copy the generated private key into the box and use
         # it for SSH by default. We have to do this because we now generate
         # random keypairs on boot, so packaged boxes would stop working
@@ -198,6 +234,15 @@ module Vagrant
             f.puts %Q[end]
           end
         end
+
+        # Check to see if package.info is a valid file and titled info.json
+        def invalid_info?
+          if @env["package.info"] != ""
+            info_path = Pathname.new(@env["package.info"])
+
+            return !info_path.file? || File.basename(info_path) != "info.json"
+          end
+        end
       end
     end
   end
