@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require 'fileutils'
 require "tempfile"
 
@@ -16,6 +19,14 @@ module Vagrant
   class Box
     include Comparable
 
+    autoload :Remote, "vagrant/box/remote"
+
+    # The required fields in a boxes `metadata.json` file
+    REQUIRED_METADATA_FIELDS = ["provider"]
+
+    # Number of seconds to wait between checks for box updates
+    BOX_UPDATE_CHECK_INTERVAL = 3600
+
     # The box name. This is the logical name used when adding the box.
     #
     # @return [String]
@@ -26,6 +37,11 @@ module Vagrant
     # @return [Symbol]
     attr_reader :provider
 
+    # This is the architecture that this box is build for.
+    #
+    # @return [String]
+    attr_reader :architecture
+
     # The version of this box.
     #
     # @return [String]
@@ -54,18 +70,24 @@ module Vagrant
     # @param [Symbol] provider The provider that this box implements.
     # @param [Pathname] directory The directory where this box exists on
     #   disk.
-    def initialize(name, provider, version, directory, **opts)
+    # @param [String] architecture Architecture the box was built for
+    # @param [String] metadata_url Metadata URL for box
+    # @param [Hook] hook A hook to apply to the box downloader, for example, for authentication
+    def initialize(name, provider, version, directory, architecture: nil, metadata_url: nil, hook: nil)
       @name      = name
       @version   = version
       @provider  = provider
       @directory = directory
-      @metadata_url = opts[:metadata_url]
+      @architecture = architecture
+      @metadata_url = metadata_url
+      @hook = hook
 
       metadata_file = directory.join("metadata.json")
       raise Errors::BoxMetadataFileNotFound, name: @name if !metadata_file.file?
 
       begin
         @metadata = JSON.parse(directory.join("metadata.json").read)
+        validate_metadata_json(@metadata)
       rescue JSON::ParserError
         raise Errors::BoxMetadataCorrupted, name: @name
       end
@@ -73,6 +95,18 @@ module Vagrant
       @logger = Log4r::Logger.new("vagrant::box")
     end
 
+    def validate_metadata_json(metadata)
+      metatdata_fields = metadata.keys
+      REQUIRED_METADATA_FIELDS.each do |field|
+        if !metatdata_fields.include?(field)
+          raise Errors::BoxMetadataMissingRequiredFields,
+            name: @name,
+            required_field: field,
+            all_fields: REQUIRED_METADATA_FIELDS.join(", ")
+        end
+      end
+    end
+
     # This deletes the box. This is NOT undoable.
     def destroy!
       # Delete the directory to delete the box.
@@ -103,6 +137,7 @@ module Vagrant
         # If all the data matches, record it
         if box_data["name"] == self.name &&
           box_data["provider"] == self.provider.to_s &&
+          box_data["architecture"] == self.architecture &&
           box_data["version"] == self.version.to_s
           results << entry
         end
@@ -117,7 +152,7 @@ module Vagrant
     #
     # @param [Hash] download_options Options to pass to the downloader.
     # @return [BoxMetadata]
-    def load_metadata(**download_options)
+    def load_metadata(download_options={})
       tf = Tempfile.new("vagrant-load-metadata")
       tf.close
 
@@ -129,8 +164,12 @@ module Vagrant
       end
 
       opts = { headers: ["Accept: application/json"] }.merge(download_options)
-      Util::Downloader.new(url, tf.path, **opts).download!
-      BoxMetadata.new(File.open(tf.path, "r"))
+      d = Util::Downloader.new(url, tf.path, opts)
+      if @hook
+        @hook.call(:authenticate_box_downloader, downloader: d)
+      end
+      d.download!
+      BoxMetadata.new(File.open(tf.path, "r"), url: url)
     rescue Errors::DownloaderError => e
       raise Errors::BoxMetadataDownloadError,
         message: e.extra_data[:message]
@@ -154,14 +193,38 @@ module Vagrant
         raise Errors::BoxUpdateNoMetadata, name: @name
       end
 
+      if download_options.delete(:automatic_check) && !automatic_update_check_allowed?
+        @logger.info("Skipping box update check")
+        return
+      end
+
       version += ", " if version
       version ||= ""
       version += "> #{@version}"
       md      = self.load_metadata(download_options)
-      newer   = md.version(version, provider: @provider)
+      newer   = md.version(version, provider: @provider, architecture: @architecture)
       return nil if !newer
 
-      [md, newer, newer.provider(@provider)]
+      [md, newer, newer.provider(@provider, @architecture)]
+    end
+
+    # Check if a box update check is allowed. Uses a file
+    # in the box data directory to track when the last auto
+    # update check was performed and returns true if the
+    # BOX_UPDATE_CHECK_INTERVAL has passed.
+    #
+    # @return [Boolean]
+    def automatic_update_check_allowed?
+      check_path = directory.join("box_update_check")
+      if check_path.exist?
+        last_check_span = Time.now.to_i - check_path.mtime.to_i
+        if last_check_span < BOX_UPDATE_CHECK_INTERVAL
+          @logger.info("box update check is under the interval threshold")
+          return false
+        end
+      end
+      FileUtils.touch(check_path)
+      true
     end
 
     # This repackages this box and outputs it to the given path.
@@ -186,13 +249,13 @@ module Vagrant
     end
 
     # Implemented for comparison with other boxes. Comparison is
-    # implemented by comparing names and providers.
+    # implemented by comparing names, providers, and architectures.
     def <=>(other)
       return super if !other.is_a?(self.class)
 
       # Comparison is done by composing the name and provider
-      "#{@name}-#{@version}-#{@provider}" <=>
-      "#{other.name}-#{other.version}-#{other.provider}"
+      "#{@name}-#{@version}-#{@provider}-#{@architecture}" <=>
+      "#{other.name}-#{other.version}-#{other.provider}-#{other.architecture}"
     end
   end
 end
