@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "digest/sha1"
 require "fileutils"
 require "monitor"
@@ -17,13 +20,15 @@ module Vagrant
     VAGRANT_SLASH = "-VAGRANTSLASH-".freeze
     VAGRANT_COLON = "-VAGRANTCOLON-".freeze
 
+    autoload :Remote, "vagrant/box_collection/remote"
+
     # The directory where the boxes in this collection are stored.
     #
     # A box collection matches a very specific folder structure that Vagrant
     # expects in order to easily manage and modify boxes. The folder structure
     # is the following:
     #
-    #     COLLECTION_ROOT/BOX_NAME/PROVIDER/metadata.json
+    #     COLLECTION_ROOT/BOX_NAME/PROVIDER/[ARCHITECTURE]/metadata.json
     #
     # Where:
     #
@@ -33,6 +38,8 @@ module Vagrant
     #     the user of Vagrant.
     #   * PROVIDER - The provider that the box was built for (VirtualBox,
     #     VMware, etc.).
+    #   * ARCHITECTURE - Optional. The architecture that the box was built
+    #     for (amd64, arm64, 386, etc.).
     #   * metadata.json - A simple JSON file that at the bare minimum
     #     contains a "provider" key that matches the provider for the
     #     box. This metadata JSON, however, can contain anything.
@@ -75,14 +82,15 @@ module Vagrant
     # @param [Boolean] force If true, any existing box with the same name
     #   and provider will be replaced.
     def add(path, name, version, **opts)
+      architecture = opts[:architecture]
       providers = opts[:providers]
       providers = Array(providers) if providers
       provider = nil
 
       # A helper to check if a box exists. We store this in a variable
       # since we call it multiple times.
-      check_box_exists = lambda do |box_formats|
-        box = find(name, box_formats, version)
+      check_box_exists = lambda do |box_formats, box_architecture|
+        box = find(name, box_formats, version, box_architecture)
         next if !box
 
         if !opts[:force]
@@ -103,12 +111,12 @@ module Vagrant
 
       with_collection_lock do
         log_provider = providers ? providers.join(", ") : "any provider"
-        @logger.debug("Adding box: #{name} (#{log_provider}) from #{path}")
+        @logger.debug("Adding box: #{name} (#{log_provider} - #{architecture.inspect}) from #{path}")
 
         # Verify the box doesn't exist early if we're given a provider. This
         # can potentially speed things up considerably since we don't need
         # to unpack any files.
-        check_box_exists.call(providers) if providers
+        check_box_exists.call(providers, architecture) if providers
 
         # Create a temporary directory since we're not sure at this point if
         # the box we're unpackaging already exists (if no provider was given)
@@ -116,7 +124,7 @@ module Vagrant
           # Extract the box into a temporary directory.
           @logger.debug("Unpacking box into temporary directory: #{temp_dir}")
           result = Util::Subprocess.execute(
-            "bsdtar", "-v", "-x", "-m", "-C", temp_dir.to_s, "-f", path.to_s)
+            "bsdtar", "--no-same-owner", "--no-same-permissions", "-v", "-x", "-m", "-S", "-s", "|\\\\\|/|", "-C", temp_dir.to_s, "-f", path.to_s)
           if result.exit_code != 0
             raise Errors::BoxUnpackageFailure,
               output: result.stderr.to_s
@@ -149,7 +157,7 @@ module Vagrant
               end
             else
               # Verify the box doesn't already exist
-              check_box_exists.call([box_provider])
+              check_box_exists.call([box_provider], architecture)
             end
 
             # We weren't given a provider, so store this one.
@@ -162,19 +170,26 @@ module Vagrant
             @logger.debug("Box directory: #{box_dir}")
 
             # This is the final directory we'll move it to
-            final_dir = box_dir.join(provider.to_s)
-            if final_dir.exist?
+            if architecture
+              arch = architecture
+              arch = Util::Platform.architecture if architecture == :auto
+              box_dir = box_dir.join(arch)
+            end
+            provider_dir = box_dir.join(provider.to_s)
+            @logger.debug("Provider directory: #{provider_dir}")
+
+            if provider_dir.exist?
               @logger.debug("Removing existing provider directory...")
-              final_dir.rmtree
+              provider_dir.rmtree
             end
 
             # Move to final destination
-            final_dir.mkpath
+            provider_dir.mkpath
 
             # Recursively move individual files from the temporary directory
             # to the final location. We do this instead of moving the entire
             # directory to avoid issues on Windows. [GH-1424]
-            copy_pairs = [[final_temp_dir, final_dir]]
+            copy_pairs = [[final_temp_dir, provider_dir]]
             while !copy_pairs.empty?
               from, to = copy_pairs.shift
               from.children(true).each do |f|
@@ -204,13 +219,13 @@ module Vagrant
       end
 
       # Return the box
-      find(name, provider, version)
+      find(name, provider, version, architecture)
     end
 
     # This returns an array of all the boxes on the system, given by
     # their name and their provider.
     #
-    # @return [Array] Array of `[name, version, provider]` of the boxes
+    # @return [Array] Array of `[name, version, provider, architecture]` of the boxes
     #   installed on this system.
     def all
       results = []
@@ -231,16 +246,52 @@ module Vagrant
             next if versiondir.basename.to_s.start_with?(".")
 
             version = versiondir.basename.to_s
+            # Ensure version of box is correct before continuing
+            if !Gem::Version.correct?(version)
+              ui = Vagrant::UI::Prefixed.new(Vagrant::UI::Colored.new, "vagrant")
+              ui.warn(I18n.t("vagrant.box_version_malformed",
+                             version: version, box_name: box_name))
+              @logger.debug("Invalid version #{version} for box #{box_name}")
+              next
+            end
 
-            versiondir.children(true).each do |provider|
-              # Verify this is a potentially valid box. If it looks
-              # correct enough then include it.
-              if provider.directory? && provider.join("metadata.json").file?
-                provider_name = provider.basename.to_s.to_sym
-                @logger.debug("Box: #{box_name} (#{provider_name})")
-                results << [box_name, version, provider_name]
-              else
-                @logger.debug("Invalid box, ignoring: #{provider}")
+            versiondir.children(true).each do |architecture_or_provider|
+              # If the entry is not a directory, it is invalid and should be ignored
+              if !architecture_or_provider.directory?
+                @logger.debug("Invalid box #{box_name} (v#{version}) - invalid item: #{architecture_or_provider}")
+                next
+              end
+
+              # Now the directory can be assumed to be the architecture
+              architecture_name = architecture_or_provider.basename.to_s.to_sym
+
+              # Cycle through directories to find providers
+              architecture_or_provider.children(true).each do |provider|
+                if !provider.directory?
+                  @logger.debug("Invalid box #{box_name} (v#{version}, #{architecture_name}) - invalid item: #{provider}")
+                  next
+                end
+
+                # If the entry contains a metadata file, add it
+                if provider.join("metadata.json").file?
+                  provider_name = provider.basename.to_s.to_sym
+                  @logger.debug("Box: #{box_name} (#{provider_name} (#{architecture_name}), #{version})")
+                  results << [box_name, version, provider_name, architecture_name]
+                end
+              end
+
+              # If the base entry contains a metadata file, then it was
+              # added prior to architecture support and is a provider directory.
+              # If it contains a metadata file, include it with the results only
+              # if an entry hasn't already been included for the local system's
+              # architecture
+              if architecture_or_provider.join("metadata.json").file?
+                provider_name = architecture_or_provider.basename.to_s.to_sym
+                if results.include?([box_name, version, provider_name, Util::Platform.architecture.to_sym])
+                  next
+                end
+                @logger.debug("Box: #{box_name} (#{provider_name}, #{version})")
+                results << [box_name, version, provider_name, nil]
               end
             end
           end
@@ -248,7 +299,7 @@ module Vagrant
       end
       # Sort the list to group like providers and properly ordered versions
       results.sort_by! do |box_result|
-        [box_result[0], box_result[2], Gem::Version.new(box_result[1])]
+        [box_result[0], box_result[2], Gem::Version.new(box_result[1]), box_result[3]]
       end
       results
     end
@@ -260,12 +311,19 @@ module Vagrant
     # @param [String] version Version constraints to adhere to. Example:
     #   "~> 1.0" or "= 1.0, ~> 1.1"
     # @return [Box] The box found, or `nil` if not found.
-    def find(name, providers, version)
+    def find(name, providers, version, box_architecture=:auto)
       providers = Array(providers)
+      architecture = box_architecture
+      architecture = Util::Platform.architecture if architecture == :auto
 
       # Build up the requirements we have
       requirements = version.to_s.split(",").map do |v|
-        Gem::Requirement.new(v.strip)
+        begin
+          Gem::Requirement.new(v.strip)
+        rescue Gem::Requirement::BadRequirementError
+          raise Errors::BoxVersionInvalid,
+                version: v.strip
+        end
       end
 
       with_collection_lock do
@@ -299,9 +357,42 @@ module Vagrant
           end
 
           versiondir = box_directory.join(version_dir_map[v.to_s])
+
           providers.each do |provider|
-            provider_dir = versiondir.join(provider.to_s)
-            next if !provider_dir.directory?
+            providerdir = versiondir.join(architecture.to_s).join(provider.to_s)
+
+            # If the architecture was automatically set to the host
+            # architecture, then a match on the architecture subdirectory
+            # or the provider directory (which is a box install prior to
+            # architecture support) is valid
+            if box_architecture == :auto
+              if !providerdir.directory?
+                providerdir = versiondir.join(provider.to_s)
+              end
+
+              if providerdir.join("metadata.json").file?
+                @logger.info("Box found: #{name} (#{provider})")
+
+                metadata_url = nil
+                metadata_url_file = box_directory.join("metadata_url")
+                metadata_url = metadata_url_file.read if metadata_url_file.file?
+
+                if metadata_url && @hook
+                  hook_env     = @hook.call(
+                    :authenticate_box_url, box_urls: [metadata_url])
+                  metadata_url = hook_env[:box_urls].first
+                end
+
+                return Box.new(
+                  name, provider, version_dir_map[v.to_s], providerdir,
+                  architecture: architecture, metadata_url: metadata_url, hook: @hook
+                )
+              end
+            end
+
+            # If there is no metadata file found, skip
+            next if !providerdir.join("metadata.json").file?
+
             @logger.info("Box found: #{name} (#{provider})")
 
             metadata_url = nil
@@ -315,8 +406,8 @@ module Vagrant
             end
 
             return Box.new(
-              name, provider, version_dir_map[v.to_s], provider_dir,
-              metadata_url: metadata_url,
+              name, provider, version_dir_map[v.to_s], providerdir,
+              architecture: architecture, metadata_url: metadata_url, hook: @hook
             )
           end
         end
