@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 # This file contains all of the internal errors in Vagrant's core
 # commands, actions, etc.
 
@@ -100,7 +103,7 @@ module Vagrant
 
       def translate_error(opts)
         return nil if !opts[:_key]
-        I18n.t("#{opts[:_namespace]}.#{opts[:_key]}", opts)
+        I18n.t("#{opts[:_namespace]}.#{opts[:_key]}", **opts)
       end
     end
 
@@ -108,6 +111,10 @@ module Vagrant
       error_key(:active_machine_with_different_provider)
     end
 
+    class AliasInvalidError < VagrantError
+      error_key(:alias_invalid_error)
+    end
+
     class BatchMultiError < VagrantError
       error_key(:batch_multi_error)
     end
@@ -156,10 +163,18 @@ module Vagrant
       error_key(:box_config_changing_box)
     end
 
+    class BoxFileNotExist < VagrantError
+      error_key(:box_file_not_exist)
+    end
+
     class BoxMetadataCorrupted < VagrantError
       error_key(:box_metadata_corrupted)
     end
 
+    class BoxMetadataMissingRequiredFields < VagrantError
+      error_key(:box_metadata_missing_required_fields)
+    end
+
     class BoxMetadataDownloadError < VagrantError
       error_key(:box_metadata_download_error)
     end
@@ -184,6 +199,10 @@ module Vagrant
       error_key(:box_not_found_with_provider)
     end
 
+    class BoxNotFoundWithProviderArchitecture < VagrantError
+      error_key(:box_not_found_with_provider_architecture)
+    end
+
     class BoxNotFoundWithProviderAndVersion < VagrantError
       error_key(:box_not_found_with_provider_and_version)
     end
@@ -196,6 +215,10 @@ module Vagrant
       error_key(:box_remove_not_found)
     end
 
+    class BoxRemoveArchitectureNotFound < VagrantError
+      error_key(:box_remove_architecture_not_found)
+    end
+
     class BoxRemoveProviderNotFound < VagrantError
       error_key(:box_remove_provider_not_found)
     end
@@ -204,6 +227,10 @@ module Vagrant
       error_key(:box_remove_version_not_found)
     end
 
+    class BoxRemoveMultiArchitecture < VagrantError
+      error_key(:box_remove_multi_architecture)
+    end
+
     class BoxRemoveMultiProvider < VagrantError
       error_key(:box_remove_multi_provider)
     end
@@ -224,6 +251,10 @@ module Vagrant
       error_key(:box_update_multi_provider)
     end
 
+    class BoxUpdateMultiArchitecture < VagrantError
+      error_key(:box_update_multi_architecture)
+    end
+
     class BoxUpdateNoMetadata < VagrantError
       error_key(:box_update_no_metadata)
     end
@@ -232,6 +263,10 @@ module Vagrant
       error_key(:failed, "vagrant.actions.box.verify")
     end
 
+    class BoxVersionInvalid < VagrantError
+      error_key(:box_version_invalid)
+    end
+
     class BundlerDisabled < VagrantError
       error_key(:bundler_disabled)
     end
@@ -240,6 +275,10 @@ module Vagrant
       error_key(:bundler_error)
     end
 
+    class SourceSpecNotFound < BundlerError
+      error_key(:source_spec_not_found)
+    end
+
     class CantReadMACAddresses < VagrantError
       error_key(:cant_read_mac_addresses)
     end
@@ -292,10 +331,22 @@ module Vagrant
       error_key(:clone_machine_not_found)
     end
 
+    class CloudInitNotFound < VagrantError
+      error_key(:cloud_init_not_found)
+    end
+
+    class CloudInitCommandFailed < VagrantError
+      error_key(:cloud_init_command_failed)
+    end
+
     class CommandDeprecated < VagrantError
       error_key(:command_deprecated)
     end
 
+    class CommandSuspendAllArgs < VagrantError
+      error_key(:command_suspend_all_arguments)
+    end
+
     class CommandUnavailable < VagrantError
       error_key(:command_unavailable)
     end
@@ -324,10 +375,18 @@ module Vagrant
       error_key(:corrupt_machine_index)
     end
 
+    class CreateIsoHostCapNotFound < VagrantError
+      error_key(:create_iso_host_cap_not_found)
+    end
+
     class DarwinMountFailed < VagrantError
       error_key(:darwin_mount_failed)
     end
 
+    class DarwinVersionFailed < VagrantError
+      error_key(:darwin_version_failed)
+    end
+
     class DestroyRequiresForce < VagrantError
       error_key(:destroy_requires_force)
     end
@@ -336,6 +395,10 @@ module Vagrant
       error_key(:dotfile_upgrade_json_error)
     end
 
+    class DownloadAlreadyInProgress < VagrantError
+      error_key(:download_already_in_progress_error)
+    end
+
     class DownloaderError < VagrantError
       error_key(:downloader_error)
     end
@@ -372,6 +435,10 @@ module Vagrant
       error_key(:home_dir_unknown_version)
     end
 
+    class HypervVirtualBoxError < VagrantError
+      error_key(:hyperv_virtualbox_error)
+    end
+
     class ForwardPortAdapterNotFound < VagrantError
       error_key(:forward_port_adapter_not_found)
     end
@@ -380,6 +447,10 @@ module Vagrant
       error_key(:auto_empty, "vagrant.actions.vm.forward_ports")
     end
 
+    class ForwardPortHostIPNotFound < VagrantError
+      error_key(:host_ip_not_found, "vagrant.actions.vm.forward_ports")
+    end
+
     class ForwardPortCollision < VagrantError
       error_key(:collision_error, "vagrant.actions.vm.forward_ports")
     end
@@ -404,6 +475,10 @@ module Vagrant
       error_key(:host_explicit_not_detected)
     end
 
+    class ISOBuildFailed < VagrantError
+      error_key(:iso_build_failed)
+    end
+
     class LinuxMountFailed < VagrantError
       error_key(:linux_mount_failed)
     end
@@ -420,6 +495,10 @@ module Vagrant
       error_key(:machine_action_locked)
     end
 
+    class MachineFolderNotAccessible < VagrantError
+      error_key(:machine_folder_not_accessible)
+    end
+
     class MachineGuestNotReady < VagrantError
       error_key(:machine_guest_not_ready)
     end
@@ -440,6 +519,10 @@ module Vagrant
       error_key(:multi_vm_target_required)
     end
 
+    class NetplanNoAvailableRenderers < VagrantError
+      error_key(:netplan_no_available_renderers)
+    end
+
     class NetSSHException < VagrantError
       error_key(:net_ssh_exception)
     end
@@ -544,6 +627,10 @@ module Vagrant
       error_key(:requires_directory, "vagrant.actions.general.package")
     end
 
+    class PackageInvalidInfo < VagrantError
+      error_key(:package_invalid_info)
+    end
+
     class PowerShellNotFound < VagrantError
       error_key(:powershell_not_found)
     end
@@ -572,6 +659,10 @@ module Vagrant
       error_key(:provider_not_found)
     end
 
+    class ProviderNotFoundSuggestion < VagrantError
+      error_key(:provider_not_found_suggestion)
+    end
+
     class ProviderNotUsable < VagrantError
       error_key(:provider_not_usable)
     end
@@ -584,6 +675,18 @@ module Vagrant
       error_key(:provisioner_winrm_unsupported)
     end
 
+    class PluginNeedsDeveloperTools < VagrantError
+      error_key(:plugin_needs_developer_tools)
+    end
+
+    class PluginMissingLibrary < VagrantError
+      error_key(:plugin_missing_library)
+    end
+
+    class PluginMissingRubyDev < VagrantError
+      error_key(:plugin_missing_ruby_dev)
+    end
+
     class PluginGemNotFound < VagrantError
       error_key(:plugin_gem_not_found)
     end
@@ -592,6 +695,10 @@ module Vagrant
       error_key(:plugin_install_license_not_found)
     end
 
+    class PluginInstallFailed < VagrantError
+      error_key(:plugin_install_failed)
+    end
+
     class PluginInstallSpace < VagrantError
       error_key(:plugin_install_space)
     end
@@ -624,6 +731,14 @@ module Vagrant
       error_key(:plugin_source_error)
     end
 
+    class PluginNoLocalError < VagrantError
+      error_key(:plugin_no_local_error)
+    end
+
+    class PluginMissingLocalError < VagrantError
+      error_key(:plugin_missing_local_error)
+    end
+
     class PushesNotDefined < VagrantError
       error_key(:pushes_not_defined)
     end
@@ -640,6 +755,10 @@ module Vagrant
       error_key(:push_strategy_not_provided)
     end
 
+    class RSyncPostCommandError < VagrantError
+      error_key(:rsync_post_command_error)
+    end
+
     class RSyncError < VagrantError
       error_key(:rsync_error)
     end
@@ -744,6 +863,10 @@ module Vagrant
       error_key(:ssh_key_type_not_supported)
     end
 
+    class SSHNoExitStatus < VagrantError
+      error_key(:ssh_no_exit_status)
+    end
+
     class SSHNoRoute < VagrantError
       error_key(:ssh_no_route)
     end
@@ -768,6 +891,26 @@ module Vagrant
       error_key(:synced_folder_unusable)
     end
 
+    class TriggersBadExitCodes < VagrantError
+      error_key(:triggers_bad_exit_codes)
+    end
+
+    class TriggersGuestNotExist < VagrantError
+      error_key(:triggers_guest_not_exist)
+    end
+
+    class TriggersGuestNotRunning < VagrantError
+      error_key(:triggers_guest_not_running)
+    end
+
+    class TriggersNoBlockGiven < VagrantError
+      error_key(:triggers_no_block_given)
+    end
+
+    class TriggersNoStageGiven < VagrantError
+      error_key(:triggers_no_stage_given)
+    end
+
     class UIExpectsTTY < VagrantError
       error_key(:ui_expects_tty)
     end
@@ -776,6 +919,34 @@ module Vagrant
       error_key(:unimplemented_provider_action)
     end
 
+    class UploadInvalidCompressionType < VagrantError
+      error_key(:upload_invalid_compression_type)
+    end
+
+    class UploadMissingExtractCapability < VagrantError
+      error_key(:upload_missing_extract_capability)
+    end
+
+    class UploadMissingTempCapability < VagrantError
+      error_key(:upload_missing_temp_capability)
+    end
+
+    class UploadSourceMissing < VagrantError
+      error_key(:upload_source_missing)
+    end
+
+    class UploaderError < VagrantError
+      error_key(:uploader_error)
+    end
+
+    class UploaderInterrupted < UploaderError
+      error_key(:uploader_interrupted)
+    end
+
+    class VagrantLocked < VagrantError
+      error_key(:vagrant_locked)
+    end
+
     class VagrantInterrupt < VagrantError
       error_key(:interrupted)
     end
@@ -792,6 +963,10 @@ module Vagrant
       error_key(:vagrantfile_name_error)
     end
 
+    class VagrantRemoteError < VagrantError
+      error_key(:vagrant_remote_error)
+    end
+
     class VagrantfileSyntaxError < VagrantError
       error_key(:vagrantfile_syntax_error)
     end
@@ -820,14 +995,34 @@ module Vagrant
       error_key(:vboxmanage_not_found_error)
     end
 
-    class VBoxManageNotFoundWSLError < VagrantError
-      error_key(:vboxmanage_not_found_wsl_error)
-    end
-
     class VirtualBoxBrokenVersion040214 < VagrantError
       error_key(:virtualbox_broken_version_040214)
     end
 
+    class VirtualBoxConfigNotFound < VagrantError
+      error_key(:virtualbox_config_not_found)
+    end
+
+    class VirtualBoxDisksDefinedExceedLimit < VagrantError
+      error_key(:virtualbox_disks_defined_exceed_limit)
+    end
+
+    class VirtualBoxDisksControllerNotFound < VagrantError
+      error_key(:virtualbox_disks_controller_not_found)
+    end
+
+    class VirtualBoxDisksNoSupportedControllers < VagrantError
+      error_key(:virtualbox_disks_no_supported_controllers)
+    end
+
+    class VirtualBoxDisksPrimaryNotFound < VagrantError
+      error_key(:virtualbox_disks_primary_not_found)
+    end
+
+    class VirtualBoxDisksUnsupportedController < VagrantError
+      error_key(:virtualbox_disks_unsupported_controller)
+    end
+
     class VirtualBoxGuestPropertyNotFound < VagrantError
       error_key(:virtualbox_guest_property_not_found)
     end
@@ -852,6 +1047,10 @@ module Vagrant
       error_key(:virtualbox_install_incomplete)
     end
 
+    class VirtualBoxMachineFolderNotFound < VagrantError
+      error_key(:virtualbox_machine_folder_not_found)
+    end
+
     class VirtualBoxNoName < VagrantError
       error_key(:virtualbox_no_name)
     end
@@ -876,6 +1075,10 @@ module Vagrant
       error_key(:virtualbox_version_empty)
     end
 
+    class VirtualBoxInvalidHostSubnet < VagrantError
+      error_key(:virtualbox_invalid_host_subnet)
+    end
+
     class VMBaseMacNotSpecified < VagrantError
       error_key(:no_base_mac, "vagrant.actions.vm.match_mac")
     end
@@ -932,6 +1135,10 @@ module Vagrant
       error_key(:power_off, "vagrant.actions.vm.export")
     end
 
+    class WinRMInvalidCommunicator < VagrantError
+      error_key(:winrm_invalid_communicator)
+    end
+
     class WSLVagrantVersionMismatch < VagrantError
       error_key(:wsl_vagrant_version_mismatch)
     end
@@ -943,5 +1150,9 @@ module Vagrant
     class WSLVirtualBoxWindowsAccessError < VagrantError
       error_key(:wsl_virtualbox_windows_access)
     end
+
+    class WSLRootFsNotFoundError < VagrantError
+      error_key(:wsl_rootfs_not_found_error)
+    end
   end
 end
