@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "log4r"
 
 require "vagrant/plugin/v1/errors"
@@ -14,6 +17,10 @@ module Vagrant
       autoload :Plugin, "vagrant/plugin/v1/plugin"
       autoload :Provider, "vagrant/plugin/v1/provider"
       autoload :Provisioner, "vagrant/plugin/v1/provisioner"
+
+      # Errors
+      autoload :Error, "vagrant/plugin/v1/error"
+      autoload :InvalidCommandName, "vagrant/plugin/v1/error"
     end
   end
 end
