@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require 'log4r'
 
 require "vagrant/util/safe_puts"
@@ -45,13 +48,13 @@ module Vagrant
         def parse_options(opts=nil)
           # make sure optparse doesn't use POSIXLY_CORRECT parsing
           ENV["POSIXLY_CORRECT"] = nil
-          
+
           # Creating a shallow copy of the arguments so the OptionParser
           # doesn't destroy the originals.
           argv = @argv.dup
 
           # Default opts to a blank optionparser if none is given
-          opts ||= OptionParser.new
+          opts ||= Vagrant::OptionParser.new
 
           # Add the help option, which must be on every command.
           opts.on_tail("-h", "--help", "Print this help") do
@@ -61,7 +64,7 @@ module Vagrant
 
           opts.parse!(argv)
           return argv
-        rescue OptionParser::InvalidOption, OptionParser::MissingArgument
+        rescue OptionParser::InvalidOption, OptionParser::MissingArgument, OptionParser::AmbiguousOption
           raise Errors::CLIInvalidOptions, help: opts.help.chomp
         end
 
@@ -107,6 +110,7 @@ module Vagrant
           }
           raise Errors::NoEnvironmentError if requires_local_env && !@env.root_path
 
+          @logger.info("getting active machines")
           # Cache the active machines outside the loop
           active_machines = @env.active_machines
 
@@ -214,6 +218,8 @@ module Vagrant
             end
           end
 
+          @logger.debug("have machine list to process")
+
           # Make sure we're only working with one VM if single target
           if options[:single_target] && machines.length != 1
             @logger.debug("Using primary machine since single target")
@@ -230,6 +236,11 @@ module Vagrant
           color_index = 0
 
           machines.each do |machine|
+            if (machine.state && machine.state.id != :not_created &&
+                !machine.index_uuid.nil? && !@env.machine_index.include?(machine.index_uuid))
+              machine.recover_machine(machine.state.id)
+            end
+
             # Set the machine color
             machine.ui.opts[:color] = color_order[color_index % color_order.length]
             color_index += 1
