@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "timeout"
 
 module Vagrant
@@ -16,6 +19,9 @@ module Vagrant
       # provide shell execution. This sort of thing will come in a future
       # version.
       class Communicator
+
+        autoload :Remote, "vagrant/plugin/remote/communicator"
+        
         # This returns true/false depending on if the given machine
         # can be communicated with using this communicator. If this returns
         # `true`, then this class will be used as the primary communication
@@ -117,6 +123,13 @@ module Vagrant
         # @see #execute
         def test(command, opts=nil)
         end
+
+        # Reset the communicator. For communicators which establish
+        # a persistent connection to the remote machine, this connection
+        # should be terminated and re-established. The communicator
+        # instance should be in a "fresh" state after calling this method.
+        def reset!
+        end
       end
     end
   end
