@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "delegate"
 require "io/console"
 require "thread"
@@ -9,6 +12,7 @@ require "vagrant/util/safe_puts"
 
 module Vagrant
   module UI
+    autoload :Remote, "vagrant/ui/remote"
     # Vagrant UIs handle communication with the outside world (typically
     # through a shell). They must respond to the following methods:
     #
@@ -45,7 +49,7 @@ module Vagrant
       end
 
       [:ask, :detail, :warn, :error, :info, :output, :success].each do |method|
-        define_method(method) do |message, *opts|
+        define_method(method) do |message, **opts|
           # Log normal console messages
           begin
             @logger.info { "#{method}: #{message}" }
@@ -53,7 +57,7 @@ module Vagrant
             # We're being called in a trap-context. Wrap in a thread.
             Thread.new do
               @logger.info { "#{method}: #{message}" }
-            end.join
+            end.join(THREAD_MAX_JOIN_TIMEOUT)
           end
         end
       end
@@ -75,11 +79,23 @@ module Vagrant
       def machine(type, *data)
         @logger.info("Machine: #{type} #{data.inspect}")
       end
+
+      # Yields self (UI)
+      # Provides a way for selectively displaying or not displaying
+      # updating content like download progress.
+      def rewriting
+        yield self
+      end
+
+      def to_proto
+        raise NotImplementedError,
+          "Vagrant::UI::Interface#to_proto"
+      end
     end
 
     # This is a UI implementation that does nothing.
     class Silent < Interface
-      def ask(*args)
+      def ask(*args, **opts)
         super
 
         # Silent can't do this, obviously.
@@ -96,7 +112,7 @@ module Vagrant
         @lock = Mutex.new
       end
 
-      def ask(*args)
+      def ask(*args, **opts)
         super
 
         # Machine-readable can't ask for input
@@ -104,8 +120,8 @@ module Vagrant
       end
 
       [:detail, :warn, :error, :info, :output, :success].each do |method|
-        define_method(method) do |message, *args, **opts|
-          machine("ui", method.to_s, message, *args, **opts)
+        define_method(method) do |message, **opts|
+          machine("ui", method.to_s, message, **opts)
         end
       end
 
@@ -128,7 +144,7 @@ module Vagrant
           @lock.synchronize do
             safe_puts("#{Time.now.utc.to_i},#{target},#{type},#{data.join(",")}")
           end
-        end.join
+        end.join(THREAD_MAX_JOIN_TIMEOUT)
       end
     end
 
@@ -148,14 +164,14 @@ module Vagrant
       # to `say`.
       [:detail, :info, :warn, :error, :output, :success].each do |method|
         class_eval <<-CODE
-          def #{method}(message, *args)
+          def #{method}(message, **opts)
             super(message)
-            say(#{method.inspect}, message, *args)
+            say(#{method.inspect}, message, **opts)
           end
         CODE
       end
 
-      def ask(message, opts=nil)
+      def ask(message, **opts)
         super(message)
 
         # We can't ask questions when the output isn't a TTY.
@@ -185,7 +201,7 @@ module Vagrant
             say(:info, "\n#{I18n.t("vagrant.stdin_cant_hide_input")}\n ", opts)
 
             # Ask again, with echo enabled
-            input = ask(message, opts.merge(echo: true))
+            input = ask(message, **opts.merge(echo: true))
           end
         end
 
@@ -220,7 +236,7 @@ module Vagrant
 
       # This method handles actually outputting a message of a given type
       # to the console.
-      def say(type, message, **opts)
+      def say(type, message, opts={})
         defaults = { new_line: true, prefix: true }
         opts     = defaults.merge(@opts).merge(opts)
 
@@ -244,11 +260,35 @@ module Vagrant
             safe_puts(format_message(type, message, **opts),
                       io: channel, printer: printer)
           end
-        end.join
+        end.join(THREAD_MAX_JOIN_TIMEOUT)
       end
 
       def format_message(type, message, **opts)
-        message
+        Util::CredentialScrubber.desensitize(message)
+      end
+    end
+
+    class NonInteractive < Basic
+      def initialize
+        super
+      end
+
+      def rewriting
+        # no-op
+      end
+
+      def report_progress(progress, total, show_parts=true)
+        # no-op
+      end
+
+      def clear_line
+        @logger.warn("Using `clear line` in a non interactive ui")
+        say(:info, "\n", opts)
+      end
+
+      def ask(*args, **opts)
+        # Non interactive can't ask for input
+        raise Errors::UIExpectsTTY
       end
     end
 
@@ -264,6 +304,14 @@ module Vagrant
         @ui     = ui
       end
 
+      def to_proto
+        @ui.to_proto
+      end
+
+      def client
+        @ui.client
+      end
+
       def initialize_copy(original)
         super
         @ui = original.instance_variable_get(:@ui).dup
@@ -274,7 +322,7 @@ module Vagrant
       # to `say`.
       [:ask, :detail, :info, :warn, :error, :output, :success].each do |method|
         class_eval <<-CODE
-          def #{method}(message, *args, **opts)
+          def #{method}(message, **opts)
             super(message)
             if !@ui.opts.key?(:bold) && !opts.key?(:bold)
               opts[:bold] = #{method.inspect} != :detail && \
@@ -283,14 +331,16 @@ module Vagrant
             if !opts.key?(:target)
               opts[:target] = @prefix
             end
-            @ui.#{method}(format_message(#{method.inspect}, message, **opts), *args, **opts)
+            @ui.#{method}(format_message(#{method.inspect}, message, **opts), **opts)
           end
         CODE
       end
 
       [:clear_line, :report_progress].each do |method|
         # By default do nothing, these aren't formatted
-        define_method(method) { |*args| @ui.send(method, *args) }
+        define_method(method) do |*args|
+          @ui.send(method, *args)
+        end
       end
 
       # For machine-readable output, set the prefix in the
@@ -320,6 +370,8 @@ module Vagrant
             type == :detail || type == :ask || opts[:prefix_spaces]
         end
 
+        message = Util::CredentialScrubber.desensitize(message)
+
         # Fast-path if there is no prefix
         return message if prefix.empty?
 
@@ -327,16 +379,28 @@ module Vagrant
         target = opts[:target] if opts.key?(:target)
         target = "#{target}:" if target != ""
 
-        # Get the lines. The first default is because if the message
-        # is an empty string, then we want to still use the empty string.
         lines = [message]
-        lines = message.split("\n") if message != ""
+        if message != ""
+          lines = [].tap do |l|
+            message.scan(/(.*?)(\n|$)/).each do |m|
+              l << m.first if m.first != "" || (m.first == "" && m.last == "\n")
+            end
+          end
+          lines << "" if message.end_with?("\n")
+        end
 
         # Otherwise, make sure to prefix every line properly
         lines.map do |line|
           "#{prefix}#{target} #{line}"
         end.join("\n")
       end
+
+      def rewriting
+        @ui.rewriting do |ui|
+          yield ui
+        end
+      end
+
     end
 
     # This is a UI implementation that outputs color for various types
