@@ -1,6 +1,11 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "rbconfig"
 require "shellwords"
+require "tempfile"
 require "tmpdir"
+require "log4r"
 
 require "vagrant/util/subprocess"
 require "vagrant/util/powershell"
@@ -11,6 +16,37 @@ module Vagrant
     # This class just contains some platform checking code.
     class Platform
       class << self
+
+        # Detect architecture of host system
+        #
+        # @return [String]
+        def architecture
+          if !defined?(@_host_architecture)
+            if ENV["VAGRANT_HOST_ARCHITECTURE"].to_s != ""
+              return @_host_architecture = ENV["VAGRANT_HOST_ARCHITECTURE"]
+            end
+
+            @_host_architecture = case RbConfig::CONFIG["target_cpu"]
+              when "x86_64"
+                "amd64"
+              when "i386"
+                "386"
+              when "arm64", "aarch64"
+                "arm64"
+              else
+                RbConfig::CONFIG["target_cpu"]
+              end
+          end
+          @_host_architecture
+        end
+
+        def logger
+          if !defined?(@_logger)
+            @_logger = Log4r::Logger.new("vagrant::util::platform")
+          end
+          @_logger
+        end
+
         def cygwin?
           if !defined?(@_cygwin)
             @_cygwin = ENV["VAGRANT_DETECTED_OS"].to_s.downcase.include?("cygwin") ||
@@ -33,11 +69,10 @@ module Vagrant
           if !defined?(@_wsl)
             @_wsl = false
             SilenceWarnings.silence! do
-              # Use PATH values to check for `/mnt/c` path indicative of WSL
-              if ENV.fetch("PATH", "").downcase.include?("/mnt/c")
-                # Validate WSL via uname output
-                uname = Subprocess.execute("uname", "-r")
-                if uname.exit_code == 0 && uname.stdout.downcase.include?("microsoft")
+              # Find 'microsoft' in /proc/version indicative of WSL
+              if File.file?('/proc/version')
+                osversion = File.open('/proc/version', &:gets)
+                if osversion.downcase.include?("microsoft")
                   @_wsl = true
                 end
               end
@@ -77,24 +112,79 @@ module Vagrant
           return @_windows_admin
         end
 
-        # Checks if the user running Vagrant on Windows is a member of the
-        # "Hyper-V Administrators" group.
+        # Checks if Hyper-V is accessible to the local user. It will check
+        # if user is in the "Hyper-V Administrators" group, is a Domain
+        # administrator, and finally will run a manual interaction with
+        # Hyper-V to determine if Hyper-V is usable for the current user.
         #
         # From: https://support.microsoft.com/en-us/kb/243330
         # SID: S-1-5-32-578
         # Name: BUILTIN\Hyper-V Administrators
+        # SID: S-1-5-21DOMAIN-512
+        # Name: Domain Admins
         #
         # @return [Boolean]
         def windows_hyperv_admin?
           return @_windows_hyperv_admin if defined?(@_windows_hyperv_admin)
 
-          @_windows_hyperv_admin = -> {
-            ps_cmd = "[System.Security.Principal.WindowsIdentity]::GetCurrent().Groups | ForEach-Object { if ($_.Value -eq 'S-1-5-32-578'){ Write-Host 'true'; break }}"
-            output = Vagrant::Util::PowerShell.execute_cmd(ps_cmd)
-            return output == 'true'
+          if ENV["VAGRANT_IS_HYPERV_ADMIN"]
+            return @_windows_hyperv_admin = true
+          end
+
+          ps_cmd = "Write-Output ([System.Security.Principal.WindowsIdentity]::GetCurrent().Groups | " \
+            "Select-Object Value | ConvertTo-JSON)"
+          output = Vagrant::Util::PowerShell.execute_cmd(ps_cmd)
+          if output
+            groups = begin
+                       JSON.load(output)
+                     rescue JSON::ParserError
+                       []
+                     end
+            admin_group = groups.detect do |g|
+              g["Value"].to_s == "S-1-5-32-578" ||
+                (g["Value"].start_with?("S-1-5-21") && g["Value"].to_s.end_with?("-512"))
+            end
+
+            if admin_group
+              return @_windows_hyperv_admin = true
+            end
+          end
+
+          ps_cmd = "$x = (Get-VMHost).Name; if($x -eq [System.Net.Dns]::GetHostName()){ Write-Output 'true'}"
+          output = Vagrant::Util::PowerShell.execute_cmd(ps_cmd)
+          result = output == "true"
+
+          return @_windows_hyperv_admin = result
+        end
+
+        # Checks if Hyper-V is enabled on the host system and returns true
+        # if enabled.
+        #
+        # @return [Boolean]
+        def windows_hyperv_enabled?
+          return @_windows_hyperv_enabled if defined?(@_windows_hyperv_enabled)
+
+          @_windows_hyperv_enabled = -> {
+            check_commands = Array.new.tap do |c|
+              c << "(Get-WindowsOptionalFeature -FeatureName Microsoft-Hyper-V-Hypervisor -Online).State"
+              c << "(Get-WindowsFeature -FeatureName Microsoft-Hyper-V-Hypervisor).State"
+            end
+            check_commands.each do |ps_cmd|
+              begin
+                output = Vagrant::Util::PowerShell.execute_cmd(ps_cmd)
+                return true if output == "Enabled"
+              rescue Errors::PowerShellInvalidVersion
+                logger.warn("Invalid PowerShell version detected during Hyper-V enable check")
+                return false
+              rescue Errors::PowerShellError
+                logger.warn("Powershell command not found or error on execution of command")
+                return false
+              end
+            end
+            return false
           }.call
 
-          return @_windows_hyperv_admin
+          return @_windows_hyperv_enabled
         end
 
         # This takes any path and converts it from a Windows path to a
@@ -155,13 +245,20 @@ module Vagrant
           return path if !cygwin?
 
           # Replace all "\" with "/", otherwise cygpath doesn't work.
-          path = path.gsub("\\", "/")
+          path = unix_windows_path(path)
 
           # Call out to cygpath and gather the result
           process = Subprocess.execute("cygpath", "-w", "-l", "-a", path.to_s)
           return process.stdout.chomp
         end
 
+        # This takes any path and converts Windows-style path separators
+        # to Unix-like path separators.
+        # @return [String]
+        def unix_windows_path(path)
+          path.gsub("\\", "/")
+        end
+
         # This checks if the filesystem is case sensitive. This is not a
         # 100% correct check, since it is possible that the temporary
         # directory runs a different filesystem than the root directory.
@@ -205,7 +302,15 @@ module Vagrant
             # Traverse each part and join it into the resulting path
             original.each do |single|
               Dir.entries(path).each do |entry|
-                if entry.downcase == single.encode('filesystem').downcase
+                begin
+                  single = single.encode("filesystem").to_s
+                rescue ArgumentError => err
+                  Vagrant.global_logger.warn("path encoding failed - part=#{single} err=#{err.class} msg=#{err}")
+                  # NOTE: Depending on the Windows environment the above
+                  # encode will generate an "input string invalid" when
+                  # attempting to encode. If that happens, continue on
+                end
+                if entry.downcase == single.downcase
                   path = path.join(entry)
                 end
               end
@@ -275,6 +380,93 @@ module Vagrant
           wsl? && !path.to_s.downcase.start_with?("/mnt/")
         end
 
+        # Compute the path to rootfs of currently active WSL.
+        #
+        # @return [String] A path to rootfs of a current WSL instance.
+        def wsl_rootfs
+          return @_wsl_rootfs if defined?(@_wsl_rootfs)
+
+          if wsl?
+            # Mark our filesystem with a temporary file having an unique name.
+            marker = Tempfile.new(Time.now.to_i.to_s)
+            logger = Log4r::Logger.new("vagrant::util::platform::wsl")
+
+            # Check for lxrun installation first
+            lxrun_path = [wsl_windows_appdata_local, "lxss"].join("\\")
+            paths = [lxrun_path]
+
+            logger.debug("checking registry for WSL installation path")
+            paths += PowerShell.execute_cmd(
+              '(Get-ChildItem HKCU:\Software\Microsoft\Windows\CurrentVersion\Lxss ' \
+                '| ForEach-Object {Get-ItemProperty $_.PSPath}).BasePath').to_s.split("\r\n").map(&:strip)
+            paths.delete_if{|path| path.to_s.empty?}
+
+            paths.each do |path|
+              # Lowercase the drive letter, skip the next symbol (which is a
+              # colon from a Windows path) and convert path to UNIX style.
+              check_path = "/mnt/#{path[0, 1].downcase}#{path[2..-1].tr('\\', '/')}/rootfs"
+              begin
+                process = Subprocess.execute("wslpath", "-u", "-a", path)
+                check_path = "#{process.stdout.chomp}/rootfs" if process.exit_code == 0
+              rescue Errors::CommandUnavailable => e
+                # pass
+              end
+
+              logger.debug("checking `#{path}` for current WSL instance")
+              begin
+                # https://blogs.msdn.microsoft.com/wsl/2016/06/15/wsl-file-system-support
+                # Current WSL instance doesn't have an access to its mount from
+                # within itself despite all others are available. That's the
+                # hacky way we're using to determine current instance.
+                # For example we have three WSL instances:
+                # A -> C:\User\USER\AppData\Local\Packages\A\LocalState\rootfs
+                # B -> C:\User\USER\AppData\Local\Packages\B\LocalState\rootfs
+                # C -> C:\User\USER\AppData\Local\Packages\C\LocalState\rootfs
+                # If we're in "A" WSL at the moment, then its path will not be
+                # accessible since it's mounted for exactly the instance we're
+                # in. All others can be opened.
+                Dir.open(check_path) do |fs|
+                  # A fallback for a case if our trick will stop working. For
+                  # that we've created a temporary file with an unique name in
+                  # a current WSL and now seeking it among all WSL.
+                  if File.exist?("#{fs.path}/#{marker.path}")
+                    @_wsl_rootfs = path
+                    break
+                  end
+                end
+              rescue Errno::EACCES
+                @_wsl_rootfs = path
+                # You can create and simultaneously run multiple WSL instances,
+                # comment out the "break", run this script within each one and
+                # it'll return only single value.
+                break
+              rescue Errno::ENOENT
+                # Warn about data discrepancy between Winreg and file system
+                # states. For the sake of justice, it's worth mentioning that
+                # it is possible only when someone will manually break WSL by
+                # removing a directory of its base path (kinda "stupid WSL
+                # uninstallation by removing hidden and system directory").
+                logger.warn("WSL instance at `#{path} is broken or no longer exists")
+              end
+              # All other exceptions have to be raised since they will mean
+              # something unpredictably terrible.
+            end
+
+            marker.close!
+
+            raise Vagrant::Errors::WSLRootFsNotFoundError if @_wsl_rootfs.nil?
+          end
+
+          # Attach the rootfs leaf to the path
+          if @_wsl_rootfs != lxrun_path
+            @_wsl_rootfs = "#{@_wsl_rootfs}\\rootfs"
+          end
+
+          logger.debug("detected `#{@_wsl_rootfs}` as current WSL instance")
+
+          @_wsl_rootfs
+        end
+
         # Convert a WSL path to the local Windows path. This is useful
         # for conversion when calling out to Windows executables from
         # the WSL
@@ -282,21 +474,33 @@ module Vagrant
         # @param [String, Pathname] path Path to convert
         # @return [String]
         def wsl_to_windows_path(path)
-          if wsl? && wsl_windows_access?
+          path = path.to_s
+          if wsl? && wsl_windows_access? && !path.match(/^[a-zA-Z]:/)
+            path = File.expand_path(path)
+            begin
+              process = Subprocess.execute("wslpath", "-w", "-a", path)
+              return process.stdout.chomp if process.exit_code == 0
+            rescue Errors::CommandUnavailable => e
+              # pass
+            end
             if wsl_path?(path)
               parts = path.split("/")
               parts.delete_if(&:empty?)
-              [wsl_windows_appdata_local, "lxss", *parts].join("\\")
+              root_path = wsl_rootfs
+              # lxrun splits home separate so we need to account
+              # for it's specialness here when we build the path
+              if root_path.end_with?("lxss") && !(["root", "home"].include?(parts.first))
+                root_path = "#{root_path}\\rootfs"
+              end
+              path = [root_path, *parts].join("\\")
             else
-              path = path.to_s.sub("/mnt/", "")
+              path = path.sub("/mnt/", "")
               parts = path.split("/")
               parts.first << ":"
               path = parts.join("\\")
-              path
             end
-          else
-            path
           end
+          path
         end
 
         # Takes a windows path and formats it to the
@@ -349,6 +553,14 @@ module Vagrant
         def wsl_windows_accessible_path
           if !defined?(@_wsl_windows_accessible_path)
             access_path = ENV["VAGRANT_WSL_WINDOWS_ACCESS_USER_HOME_PATH"]
+            if access_path.to_s.empty?
+              begin
+                process = Subprocess.execute("wslpath", "-u", "-a", wsl_windows_home)
+                access_path = process.stdout.chomp if process.exit_code == 0
+              rescue Errors::CommandUnavailable => e
+                # pass
+              end
+            end
             if access_path.to_s.empty?
               access_path = wsl_windows_home.gsub("\\", "/").sub(":", "")
               access_path[0] = access_path[0].downcase
@@ -368,6 +580,42 @@ module Vagrant
             path.to_s.start_with?(wsl_windows_accessible_path.to_s)
         end
 
+        # Mount pattern for extracting local mount information
+        MOUNT_PATTERN = /^(?<device>.+?) on (?<mount>.+?) type (?<type>.+?) \((?<options>.+)\)/.freeze
+
+        # Get list of local mount paths that are DrvFs file systems
+        #
+        # @return [Array<String>]
+        # @todo(chrisroberts): Constantize types for check
+        def wsl_drvfs_mounts
+          if !defined?(@_wsl_drvfs_mounts)
+            @_wsl_drvfs_mounts = []
+            if wsl?
+              result = Util::Subprocess.execute("mount")
+              result.stdout.each_line do |line|
+                info = line.match(MOUNT_PATTERN)
+                if info && (info[:type] == "drvfs" || info[:type] == "9p")
+                  @_wsl_drvfs_mounts << info[:mount]
+                end
+              end
+            end
+          end
+          @_wsl_drvfs_mounts
+        end
+
+        # Check if given path is located on DrvFs file system
+        #
+        # @param [String, Pathname] path Path to check
+        # @return [Boolean]
+        def wsl_drvfs_path?(path)
+          if wsl?
+            wsl_drvfs_mounts.each do |mount_path|
+              return true if path.to_s.start_with?(mount_path)
+            end
+          end
+          false
+        end
+
         # If running within the Windows Subsystem for Linux, this will provide
         # simple setup to allow sharing of the user's VAGRANT_HOME directory
         # within the subsystem
@@ -448,9 +696,9 @@ module Vagrant
         def wsl_validate_matching_vagrant_versions!
           valid = false
           if Util::Which.which("vagrant.exe")
-            result = Util::Subprocess.execute("vagrant.exe", "version")
+            result = Util::Subprocess.execute("vagrant.exe", "--version")
             if result.exit_code == 0
-              windows_version = result.stdout.match(/Installed Version: (?<version>[\w.-]+)/)
+              windows_version = result.stdout.match(/Vagrant (?<version>[\w.-]+)/)
               if windows_version
                 windows_version = windows_version[:version].strip
                 valid = windows_version == Vagrant::VERSION
