@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "log4r"
 
 module Vagrant
@@ -11,7 +14,7 @@ module Vagrant
       # http://blog.codefront.net/2008/01/14/retrying-code-blocks-in-ruby-on-exceptions-whatever/
       def retryable(opts=nil)
         logger = nil
-        opts   = { :tries => 1, :on => Exception }.merge(opts || {})
+        opts   = { tries: 1, on: Exception }.merge(opts || {})
 
         begin
           return yield
