@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "fileutils"
 require "pathname"
 require "rubygems"
@@ -17,21 +20,12 @@ module VagrantPlugins
         end
 
         def call(env)
-          # Get the list of installed plugins according to the state file
-          installed = env[:plugin_state_file].installed_plugins.keys
-
-          # If the plugin we're trying to license doesn't exist in the
-          # state file, then it is an error.
-          if !installed.include?(env[:plugin_name])
-            raise Vagrant::Errors::PluginNotFound, :name => env[:plugin_name]
-          end
-
           # Verify the license file exists
           license_file = Pathname.new(env[:plugin_license_path])
           if !license_file.file?
             raise Vagrant::Errors::PluginInstallLicenseNotFound,
-              :name => env[:plugin_name],
-              :path => license_file.to_s
+              name: env[:plugin_name],
+              path: license_file.to_s
           end
 
           # Copy it in.
@@ -39,12 +33,12 @@ module VagrantPlugins
           @logger.info("Copying license from: #{license_file}")
           @logger.info("Copying license to: #{final_path}")
           env[:ui].info(I18n.t("vagrant.commands.plugin.installing_license",
-                               :name => env[:plugin_name]))
+                               name: env[:plugin_name]))
           FileUtils.cp(license_file, final_path)
 
           # Installed!
           env[:ui].success(I18n.t("vagrant.commands.plugin.installed_license",
-                                 :name => env[:plugin_name]))
+                                 name: env[:plugin_name]))
 
           @app.call(env)
         end
