@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require 'optparse'
 
 require_relative "base"
@@ -14,12 +17,12 @@ module VagrantPlugins
           # Parse the options
           argv = parse_options(opts)
           return if !argv
-          raise Vagrant::Errors::CLIInvalidUsage, :help => opts.help.chomp if argv.length < 2
+          raise Vagrant::Errors::CLIInvalidUsage, help: opts.help.chomp if argv.length < 2
 
           # License the plugin
           action(Action.action_license, {
-            :plugin_license_path => argv[1],
-            :plugin_name         => argv[0]
+            plugin_license_path: argv[1],
+            plugin_name:         argv[0]
           })
 
           # Success, exit status 0
