@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "vagrant"
 
 module VagrantPlugins
@@ -16,7 +19,5 @@ DESC
     end
 
     autoload :Action, File.expand_path("../action", __FILE__)
-    autoload :GemHelper, File.expand_path("../gem_helper", __FILE__)
-    autoload :StateFile, File.expand_path("../state_file", __FILE__)
   end
 end
