@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require 'optparse'
 
 require "vagrant"
@@ -23,6 +26,9 @@ module VagrantPlugins
           o.banner = "Usage: vagrant reload [vm-name]"
           o.separator ""
           build_start_options(o, options)
+          o.on("-f", "--force", "Force shut down (equivalent of pulling power)") do |f|
+            options[:force_halt] = f
+          end
         end
 
         # Parse the options
