@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require 'optparse'
 
 require "vagrant/util/safe_puts"
@@ -55,6 +58,8 @@ module VagrantPlugins
             proxy_command: ssh_info[:proxy_command],
             ssh_command:   ssh_info[:ssh_command],
             forward_env:   ssh_info[:forward_env],
+            config:        ssh_info[:config],
+            disable_deprecated_algorithms: ssh_info[:disable_deprecated_algorithms]
           }
 
           # Render the template and output directly to STDOUT
