@@ -1,3 +1,7 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
+require 'etc'
 require 'logger'
 require 'pathname'
 require 'stringio'
@@ -19,6 +23,7 @@ module VagrantPlugins
   module CommunicatorSSH
     # This class provides communication with the VM via SSH.
     class Communicator < Vagrant.plugin("2", :communicator)
+      READY_COMMAND=""
       # Marker for start of PTY enabled command output
       PTY_DELIM_START = "bccbb768c119429488cfd109aacea6b5-pty"
       # Marker for end of PTY enabled command output
@@ -64,18 +69,20 @@ module VagrantPlugins
           while true
             ssh_info = @machine.ssh_info
             break if ssh_info
-            sleep 0.5
+            sleep(0.5)
           end
 
           # Got it! Let the user know what we're connecting to.
-          @machine.ui.detail("SSH address: #{ssh_info[:host]}:#{ssh_info[:port]}")
-          @machine.ui.detail("SSH username: #{ssh_info[:username]}")
-          ssh_auth_type = "private key"
-          ssh_auth_type = "password" if ssh_info[:password]
-          @machine.ui.detail("SSH auth method: #{ssh_auth_type}")
+          if !@ssh_info_notification
+            @machine.ui.detail("SSH address: #{ssh_info[:host]}:#{ssh_info[:port]}")
+            @machine.ui.detail("SSH username: #{ssh_info[:username]}")
+            ssh_auth_type = "private key"
+            ssh_auth_type = "password" if ssh_info[:password]
+            @machine.ui.detail("SSH auth method: #{ssh_auth_type}")
+            @ssh_info_notification = true
+          end
 
-          last_message = nil
-          last_message_repeat_at = 0
+          previous_messages = {}
           while true
             message  = nil
             begin
@@ -122,14 +129,13 @@ module VagrantPlugins
             if message
               message_at   = Time.now.to_f
               show_message = true
-              if last_message == message
-                show_message = (message_at - last_message_repeat_at) > 10.0
+              if previous_messages[message]
+                show_message = (message_at - previous_messages[message]) > 10.0
               end
 
               if show_message
                 @machine.ui.detail("Warning: #{message} Retrying...")
-                last_message = message
-                last_message_repeat_at = message_at
+                previous_messages[message] = message_at
               end
             end
           end
@@ -153,7 +159,7 @@ module VagrantPlugins
         end
 
         # Verify the shell is valid
-        if execute("", error_check: false) != 0
+        if execute(self.class.const_get(:READY_COMMAND), error_check: false) != 0
           raise Vagrant::Errors::SSHInvalidShell
         end
 
@@ -166,6 +172,7 @@ module VagrantPlugins
 
         # If we used a password, then insert the insecure key
         ssh_info = @machine.ssh_info
+        return if ssh_info.nil?
         insert   = ssh_info[:password] && ssh_info[:private_key_path].empty?
         ssh_info[:private_key_path].each do |pk|
           if insecure_key?(pk)
@@ -181,7 +188,29 @@ module VagrantPlugins
             @machine.guest.capability?(:remove_public_key)
           raise Vagrant::Errors::SSHInsertKeyUnsupported if !cap
 
-          _pub, priv, openssh = Vagrant::Util::Keypair.create
+          # Check for supported key type
+          key_type = catch(:key_type) do
+            begin
+              Vagrant::Util::Keypair::PREFER_KEY_TYPES.each do |type_name, type|
+                throw :key_type, type if supports_key_type?(type_name)
+              end
+              nil
+            rescue => err
+              @logger.warn("Failed to check key types server supports: #{err}")
+              nil
+            end
+          end
+
+          @logger.debug("Detected key type for new private key: #{key_type}")
+
+          # If no key type was discovered, default to rsa
+          if key_type.nil?
+            @logger.debug("Failed to detect supported key type, defaulting to rsa")
+            key_type = :rsa
+          end
+
+          @logger.info("Creating new ssh keypair (type: #{key_type.inspect})")
+          _pub, priv, openssh = Vagrant::Util::Keypair.create(type: key_type)
 
           @logger.info("Inserting key to avoid password: #{openssh}")
           @machine.ui.detail("\n"+I18n.t("vagrant.inserting_random_key"))
@@ -193,6 +222,11 @@ module VagrantPlugins
             f.write(priv)
           end
 
+          # Adjust private key file permissions if host provides capability
+          if @machine.env.host.capability?(:set_ssh_key_permissions)
+            @machine.env.host.capability(:set_ssh_key_permissions, @machine.data_dir.join("private_key"))
+          end
+
           # Remove the old key if it exists
           @machine.ui.detail(I18n.t("vagrant.inserting_remove_key"))
           @machine.guest.capability(
@@ -220,6 +254,7 @@ module VagrantPlugins
           command:     command,
           shell:       nil,
           sudo:        false,
+          force_raw:   false
         }.merge(opts || {})
 
         opts[:good_exit] = Array(opts[:good_exit])
@@ -231,6 +266,7 @@ module VagrantPlugins
           shell_opts = {
             sudo: opts[:sudo],
             shell: opts[:shell],
+            force_raw: opts[:force_raw]
           }
 
           shell_execute(connection, command, **shell_opts) do |type, data|
@@ -283,15 +319,49 @@ module VagrantPlugins
       def upload(from, to)
         @logger.debug("Uploading: #{from} to #{to}")
 
-        scp_connect do |scp|
-          if File.directory?(from)
-            # Recurisvely upload directories
-            scp.upload!(from, to, recursive: true)
+        if File.directory?(from)
+          if from.end_with?(".")
+            @logger.debug("Uploading directory contents of: #{from}")
+            from = from.sub(/\.$/, "")
           else
-            # Open file read only to fix issue [GH-1036]
-            scp.upload!(File.open(from, "r"), to)
+            @logger.debug("Uploading full directory container of: #{from}")
+            to = File.join(to, File.basename(File.expand_path(from)))
           end
         end
+
+        scp_connect do |scp|
+          uploader = lambda do |path, remote_dest=nil|
+            if File.directory?(path)
+              dest = File.join(to, path.sub(/^#{Regexp.escape(from)}/, ""))
+              create_remote_directory(dest)
+              Dir.new(path).each do |entry|
+                next if entry == "." || entry == ".."
+                full_path = File.join(path, entry)
+                create_remote_directory(dest)
+                uploader.call(full_path, dest)
+              end
+            else
+              if remote_dest
+                dest = File.join(remote_dest, File.basename(path))
+              else
+                dest = to
+                if to.end_with?(File::SEPARATOR)
+                  dest = File.join(to, File.basename(path))
+                end
+              end
+              @logger.debug("Ensuring remote directory exists for destination upload")
+              create_remote_directory(File.dirname(dest))
+              @logger.debug("Uploading file #{path} to remote #{dest}")
+              upload_file = File.open(path, "rb")
+              begin
+                scp.upload!(upload_file, dest)
+              ensure
+                upload_file.close
+              end
+            end
+          end
+          uploader.call(from)
+        end
       rescue RuntimeError => e
         # Net::SCP raises a runtime error for this so the only way we have
         # to really catch this exception is to check the message to see if
@@ -305,6 +375,20 @@ module VagrantPlugins
           to: to.to_s
       end
 
+      def reset!
+        if @connection
+          @connection.close
+          @connection = nil
+        end
+        @ssh_info_notification = true # suppress ssh info output
+        wait_for_ready(5)
+      end
+
+      def generate_environment_export(env_key, env_value)
+        template = machine_config_ssh.export_command_template
+        template.sub("%ENV_KEY%", env_key).sub("%ENV_VALUE%", env_value) + "\n"
+      end
+
       protected
 
       # Opens an SSH connection and yields it to a block.
@@ -349,14 +433,6 @@ module VagrantPlugins
         auth_methods << "publickey" if ssh_info[:private_key_path]
         auth_methods << "password" if ssh_info[:password]
 
-        # yanked directly from ruby's Net::SSH, but with `none` last
-        # TODO: Remove this once Vagrant has updated its dependency on Net:SSH
-        # to be > 4.1.0, which should include this fix.
-        cipher_array = Net::SSH::Transport::Algorithms::ALGORITHMS[:encryption].dup
-        if cipher_array.delete("none")
-          cipher_array.push("none")
-        end
-
         # Build the options we'll use to initiate the connection via Net::SSH
         common_connect_opts = {
           auth_methods:          auth_methods,
@@ -367,10 +443,9 @@ module VagrantPlugins
           verify_host_key:       ssh_info[:verify_host_key],
           password:              ssh_info[:password],
           port:                  ssh_info[:port],
-          timeout:               15,
+          timeout:               ssh_info[:connect_timeout],
           user_known_hosts_file: [],
-          verbose:               :debug,
-          encryption:            cipher_array,
+          verbose:               :debug
         }
 
         # Connect to SSH, giving it a few tries
@@ -398,6 +473,19 @@ module VagrantPlugins
                   connect_opts[:proxy] = Net::SSH::Proxy::Command.new(ssh_info[:proxy_command])
                 end
 
+                if ssh_info[:config]
+                  connect_opts[:config] = ssh_info[:config]
+                end
+
+                if ssh_info[:remote_user]
+                  connect_opts[:remote_user] = ssh_info[:remote_user]
+                end
+
+                if @machine.config.ssh.keep_alive
+                  connect_opts[:keepalive] = true
+                  connect_opts[:keepalive_interval] = 5
+                end
+                
                 @logger.info("Attempting to connect to SSH...")
                 @logger.info("  - Host: #{ssh_info[:host]}")
                 @logger.info("  - Port: #{ssh_info[:port]}")
@@ -406,7 +494,7 @@ module VagrantPlugins
                 @logger.info("  - Key Path: #{ssh_info[:private_key_path]}")
                 @logger.debug("  - connect_opts: #{connect_opts}")
 
-                Net::SSH.start(ssh_info[:host], ssh_info[:username], connect_opts)
+                Net::SSH.start(ssh_info[:host], ssh_info[:username], **connect_opts)
               ensure
                 # Make sure we output the connection log
                 @logger.debug("== Net-SSH connection debug-level log START ==")
@@ -549,14 +637,14 @@ module VagrantPlugins
                 stderr_data_buffer << data
                 marker_index = stderr_data_buffer.index(CMD_GARBAGE_MARKER)
                 if marker_index
-                  marker_found = true
+                  stderr_marker_found = true
                   stderr_data_buffer.slice!(0, marker_index + CMD_GARBAGE_MARKER.size)
                   data.replace(stderr_data_buffer)
-                  data_buffer = nil
+                  stderr_data_buffer = nil
                 end
               end
 
-              if block_given? && marker_found && !data.empty?
+              if block_given? && stderr_marker_found && !data.empty?
                 yield :stderr, data
               end
             end
@@ -571,7 +659,7 @@ module VagrantPlugins
             end
 
             # Set the terminal
-            ch2.send_data generate_environment_export("TERM", "vt100")
+            ch2.send_data(generate_environment_export("TERM", "vt100"))
 
             # Set SSH_AUTH_SOCK if we are in sudo and forwarding agent.
             # This is to work around often misconfigured boxes where
@@ -587,14 +675,14 @@ module VagrantPlugins
               if auth_socket != ""
                 # Make sure we only read the last line which should be
                 # the $SSH_AUTH_SOCK env var we printed.
-                auth_socket = auth_socket.split("\n").last.chomp
+                auth_socket = auth_socket.split("\n").last.to_s.chomp
               end
 
               if auth_socket == ""
                 @logger.warn("No SSH_AUTH_SOCK found despite forward_agent being set.")
               else
                 @logger.info("Setting SSH_AUTH_SOCK remotely: #{auth_socket}")
-                ch2.send_data generate_environment_export("SSH_AUTH_SOCK", auth_socket)
+                ch2.send_data(generate_environment_export("SSH_AUTH_SOCK", auth_socket))
               end
             end
 
@@ -612,11 +700,11 @@ module VagrantPlugins
               data += "printf #{PTY_DELIM_END}\n"
               data += "exit $exitcode\n"
               data = data.force_encoding('ASCII-8BIT')
-              ch2.send_data data
+              ch2.send_data(data)
             else
-              ch2.send_data "printf '#{CMD_GARBAGE_MARKER}'\n(>&2 printf '#{CMD_GARBAGE_MARKER}')\n#{command}\n".force_encoding('ASCII-8BIT')
+              ch2.send_data("printf '#{CMD_GARBAGE_MARKER}'\n(>&2 printf '#{CMD_GARBAGE_MARKER}')\n#{command}\n".force_encoding('ASCII-8BIT'))
               # Remember to exit or this channel will hang open
-              ch2.send_data "exit\n"
+              ch2.send_data("exit\n")
             end
 
             # Send eof to let server know we're done
@@ -625,21 +713,6 @@ module VagrantPlugins
         end
 
         begin
-          keep_alive = nil
-
-          if machine_config_ssh.keep_alive
-            # Begin sending keep-alive packets while we wait for the script
-            # to complete. This avoids connections closing on long-running
-            # scripts.
-            keep_alive = Thread.new do
-              loop do
-                sleep 5
-                @logger.debug("Sending SSH keep-alive...")
-                connection.send_global_request("keep-alive@openssh.com")
-              end
-            end
-          end
-
           # Wait for the channel to complete
           begin
             channel.wait
@@ -653,9 +726,6 @@ module VagrantPlugins
           rescue Net::SSH::Disconnect
             raise Vagrant::Errors::SSHDisconnected
           end
-        ensure
-          # Kill the keep-alive thread
-          keep_alive.kill if keep_alive
         end
 
         # If we're in a PTY, we now finally parse the output
@@ -672,6 +742,11 @@ module VagrantPlugins
           yield :stdout, data if block_given?
         end
 
+        if !exit_status
+          @logger.debug("Exit status: #{exit_status.inspect}")
+          raise Vagrant::Errors::SSHNoExitStatus
+        end
+
         # Return the final exit status
         return exit_status
       end
@@ -698,18 +773,47 @@ module VagrantPlugins
       def insecure_key?(path)
         return false if !path
         return false if !File.file?(path)
-        source_path = Vagrant.source_root.join("keys", "vagrant")
-        return File.read(path).chomp == source_path.read.chomp
+        Dir.glob(Vagrant.source_root.join("keys", "vagrant.key.*")).any? do |source_path|
+          File.read(path).chomp == File.read(source_path).chomp
+        end
       end
 
-      def generate_environment_export(env_key, env_value)
-        template = machine_config_ssh.export_command_template
-        template.sub("%ENV_KEY%", env_key).sub("%ENV_VALUE%", env_value) + "\n"
+      def create_remote_directory(dir)
+        execute("mkdir -p \"#{dir}\"")
       end
 
       def machine_config_ssh
         @machine.config.ssh
       end
+
+      protected
+
+      # Check if server supports given key type
+      #
+      # @param [String, Symbol] type Key type
+      # @return [Boolean]
+      # @note This does not use a stable API and may be subject
+      # to unexpected breakage on net-ssh updates
+      def supports_key_type?(type)
+        if @connection.nil?
+          raise Vagrant::Errors::SSHNotReady
+        end
+        server_data = @connection.
+          transport&.
+          algorithms&.
+          instance_variable_get(:@server_data)
+        if server_data.nil?
+          @logger.warn("No server data available for key type support check")
+          return false
+        end
+        if !server_data.is_a?(Hash)
+          @logger.warn("Server data is not expected type (expecting Hash, got #{server_data.class})")
+          return false
+        end
+
+        @logger.debug("server data used for host key support check: #{server_data.inspect}")
+        server_data[:host_key].include?(type.to_s)
+      end
     end
   end
 end
