@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 module VagrantPlugins
   module CommunicatorWinRM
     class Config < Vagrant.plugin("2", :config)
@@ -13,6 +16,7 @@ module VagrantPlugins
       attr_accessor :ssl_peer_verification
       attr_accessor :execution_time_limit
       attr_accessor :basic_auth_only
+      attr_accessor :codepage
 
       def initialize
         @username               = UNSET_VALUE
@@ -27,6 +31,7 @@ module VagrantPlugins
         @ssl_peer_verification  = UNSET_VALUE
         @execution_time_limit   = UNSET_VALUE
         @basic_auth_only        = UNSET_VALUE
+        @codepage               = UNSET_VALUE
       end
 
       def finalize!
@@ -43,6 +48,7 @@ module VagrantPlugins
         @ssl_peer_verification = true if @ssl_peer_verification == UNSET_VALUE
         @execution_time_limit = "PT2H"   if @execution_time_limit == UNSET_VALUE
         @basic_auth_only = false    if @basic_auth_only == UNSET_VALUE
+        @codepage = nil        if @codepage == UNSET_VALUE
       end
 
       def validate(machine)
@@ -53,7 +59,7 @@ module VagrantPlugins
         errors << "winrm.port cannot be nil."        if @port.nil?
         errors << "winrm.guest_port cannot be nil."  if @guest_port.nil?
         errors << "winrm.max_tries cannot be nil."   if @max_tries.nil?
-        errors << "winrm.retry_delay cannot be nil." if @max_tries.nil?
+        errors << "winrm.retry_delay cannot be nil." if @retry_delay.nil?
         errors << "winrm.timeout cannot be nil."     if @timeout.nil?
         errors << "winrm.execution_time_limit cannot be nil."     if @execution_time_limit.nil?
         unless @ssl_peer_verification == true || @ssl_peer_verification == false
