@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "ipaddr"
 require "socket"
 require "tempfile"
@@ -9,10 +12,12 @@ module VagrantPlugins
     module Cap
       class ConfigureNetworks
         include Vagrant::Util
+        extend Vagrant::Util::GuestInspection::Linux
 
         def self.configure_networks(machine, networks)
           comm = machine.communicate
           commands = []
+          uses_systemd_networkd = systemd_networkd?(comm)
 
           interfaces = machine.guest.capability(:network_interfaces)
           networks.each.with_index do |network, i|
@@ -25,9 +30,15 @@ module VagrantPlugins
               network[:netmask] = (32-Math.log2((IPAddr.new(network[:netmask], Socket::AF_INET).to_i^0xffffffff)+1)).to_i
             end
 
-            entry = TemplateRenderer.render("guests/arch/network_#{network[:type]}",
-              options: network,
-            )
+            if uses_systemd_networkd
+              entry = TemplateRenderer.render("guests/arch/systemd_networkd/network_#{network[:type]}",
+                options: network,
+              )
+            else
+              entry = TemplateRenderer.render("guests/arch/default_network/network_#{network[:type]}",
+                options: network,
+              )
+            end
 
             remote_path = "/tmp/vagrant-network-#{network[:device]}-#{Time.now.to_i}-#{i}"
 
@@ -39,13 +50,22 @@ module VagrantPlugins
               comm.upload(f.path, remote_path)
             end
 
-            commands << <<-EOH.gsub(/^ {14}/, '').rstrip
-              # Configure #{network[:device]}
-              mv '#{remote_path}' '/etc/netctl/#{network[:device]}' &&
-              ip link set '#{network[:device]}' down &&
-              netctl restart '#{network[:device]}' &&
-              netctl enable '#{network[:device]}'
-            EOH
+            if uses_systemd_networkd
+              commands << <<-EOH.gsub(/^ {16}/, '').rstrip
+                # Configure #{network[:device]}
+                chmod 0644 '#{remote_path}' &&
+                mv '#{remote_path}' '/etc/systemd/network/#{network[:device]}.network' &&
+                networkctl reload
+              EOH
+            else
+              commands << <<-EOH.gsub(/^ {16}/, '').rstrip
+                # Configure #{network[:device]}
+                mv '#{remote_path}' '/etc/netctl/#{network[:device]}' &&
+                ip link set '#{network[:device]}' down &&
+                netctl restart '#{network[:device]}' &&
+                netctl enable '#{network[:device]}'
+              EOH
+            end
           end
 
           # Run all the network modification commands in one communicator call.
