@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "vagrant"
 
 module VagrantPlugins
@@ -11,6 +14,21 @@ module VagrantPlugins
         Guest
       end
 
+      guest_capability(:bsd, :create_tmp_path) do
+        require_relative "cap/file_system"
+        Cap::FileSystem
+      end
+
+      guest_capability(:bsd, :decompress_tgz) do
+        require_relative "cap/file_system"
+        Cap::FileSystem
+      end
+
+      guest_capability(:bsd, :decompress_zip) do
+        require_relative "cap/file_system"
+        Cap::FileSystem
+      end
+
       guest_capability(:bsd, :halt) do
         require_relative "cap/halt"
         Cap::Halt
@@ -27,8 +45,8 @@ module VagrantPlugins
       end
 
       guest_capability(:bsd, :mount_virtualbox_shared_folder) do
-        require_relative "cap/virtualbox"
-        Cap::VirtualBox
+        require_relative "cap/mount_virtualbox_shared_folder"
+        Cap::MountVirtualBoxSharedFolder
       end
 
       guest_capability(:bsd, :remove_public_key) do
