@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "vagrant"
 
 module VagrantPlugins
@@ -6,22 +9,22 @@ module VagrantPlugins
       name "CoreOS guest"
       description "CoreOS guest support."
 
-      guest("coreos", "linux") do
-        require File.expand_path("../guest", __FILE__)
+      guest(:coreos, :linux) do
+        require_relative "guest"
         Guest
       end
 
-      guest_capability("coreos", "change_host_name") do
+      guest_capability(:coreos, :change_host_name) do
         require_relative "cap/change_host_name"
         Cap::ChangeHostName
       end
 
-      guest_capability("coreos", "configure_networks") do
+      guest_capability(:coreos, :configure_networks) do
         require_relative "cap/configure_networks"
         Cap::ConfigureNetworks
       end
 
-      guest_capability("coreos", "docker_daemon_running") do
+      guest_capability(:coreos, :docker_daemon_running) do
         require_relative "cap/docker"
         Cap::Docker
       end
