@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "vagrant"
 
 module VagrantPlugins
@@ -21,6 +24,11 @@ module VagrantPlugins
         Cap::ConfigureNetworks
       end
 
+      guest_capability(:openbsd, :halt) do
+        require_relative "cap/halt"
+        Cap::Halt
+      end
+
       guest_capability(:openbsd, :rsync_install) do
         require_relative "cap/rsync"
         Cap::RSync
