@@ -1,8 +1,11 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 module VagrantPlugins
   module GuestPhoton
     class Guest < Vagrant.plugin("2", :guest)
       def detect?(machine)
-        machine.communicate.test("cat /etc/photon-release | grep 'VMware Photon Linux'")
+        machine.communicate.test("cat /etc/photon-release | grep 'VMware Photon'")
       end
     end
   end
