@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "vagrant"
 
 module VagrantPlugins
@@ -6,48 +9,44 @@ module VagrantPlugins
       name "Red Hat Enterprise Linux guest"
       description "Red Hat Enterprise Linux guest support."
 
-      guest("redhat", "linux") do
-        require File.expand_path("../guest", __FILE__)
+      guest(:redhat, :linux) do
+        require_relative "guest"
         Guest
       end
 
-      guest_capability("redhat", "change_host_name") do
+      guest_capability(:redhat, :change_host_name) do
         require_relative "cap/change_host_name"
         Cap::ChangeHostName
       end
 
-      guest_capability("redhat", "configure_networks") do
+      guest_capability(:redhat, :configure_networks) do
         require_relative "cap/configure_networks"
         Cap::ConfigureNetworks
       end
 
-      guest_capability("redhat", "flavor") do
+      guest_capability(:redhat, :flavor) do
         require_relative "cap/flavor"
         Cap::Flavor
       end
 
-      guest_capability("redhat", "network_scripts_dir") do
+      guest_capability(:redhat, :network_scripts_dir) do
         require_relative "cap/network_scripts_dir"
         Cap::NetworkScriptsDir
       end
 
-      guest_capability("redhat", "nfs_client_install") do
+      guest_capability(:redhat, :nfs_client_install) do
         require_relative "cap/nfs_client"
         Cap::NFSClient
       end
 
-      guest_capability("redhat", "nfs_client_installed") do
-        require_relative "cap/nfs_client"
-        Cap::NFSClient
-      end
-
-      guest_capability("redhat", "rsync_install") do
+      guest_capability(:redhat, :rsync_install) do
         require_relative "cap/rsync"
         Cap::RSync
       end
 
-      def self.dnf?(machine)
-        machine.communicate.test("/usr/bin/which -s dnf")
+      guest_capability(:redhat, :smb_install) do
+        require_relative "cap/smb"
+        Cap::SMB
       end
     end
   end
