@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 module VagrantPlugins
   module GuestSmartos
     module Cap
@@ -10,7 +13,7 @@ module VagrantPlugins
           # does not exist in /etc/user_attr. TODO
           begin
             machine.communicate.execute(
-              "#{machine.config.smartos.suexec_cmd} /usr/sbin/shutdown -y -i5 -g0")
+              "#{machine.config.smartos.suexec_cmd} /usr/sbin/poweroff")
           rescue IOError, Vagrant::Errors::SSHDisconnected
             # Ignore, this probably means connection closed because it
             # shut down.
