@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 module VagrantPlugins
   module GuestSolaris
     module Cap
@@ -11,7 +14,7 @@ module VagrantPlugins
           begin
             machine.communicate.execute(
               "#{machine.config.solaris.suexec_cmd} /usr/sbin/shutdown -y -i5 -g0")
-          rescue IOError
+          rescue IOError, Vagrant::Errors::SSHDisconnected
             # Ignore, this probably means connection closed because it
             # shut down.
           end
