@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "vagrant/util"
 require "vagrant/util/retryable"
 
@@ -19,7 +22,7 @@ module VagrantPlugins
         end
 
         def self.nfs_exports_template(environment)
-          "nfs/exports_freebsd"
+          "nfs/exports_bsd"
         end
 
         def self.nfs_restart_command(environment)
