@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "shellwords"
 require "vagrant/util"
 require "vagrant/util/shell_quote"
@@ -67,6 +70,7 @@ module VagrantPlugins
 
           nfs_opts_setup(folders)
           folders = folder_dupe_check(folders)
+          ips = ips.uniq
           output = Vagrant::Util::TemplateRenderer.render('nfs/exports_linux',
                                            uuid: id,
                                            ips: ips,
@@ -77,7 +81,7 @@ module VagrantPlugins
           sleep 0.5
 
           nfs_cleanup("#{Process.uid} #{id}")
-          output = "#{nfs_exports_content}\n#{output}"
+          output = nfs_exports_content + output
           nfs_write_exports(output)
 
           if nfs_running?(nfs_check_command)
@@ -93,7 +97,7 @@ module VagrantPlugins
               "systemctl --no-pager --no-legend --plain list-unit-files --all --type=service " \
                 "| grep #{nfs_service_name_systemd}").exit_code == 0
           else
-            Vagrant::Util::Subprocess.execute("modinfo", "nfsd").exit_code == 0 ||
+            Vagrant::Util::Subprocess.execute(modinfo_path, "nfsd").exit_code == 0 ||
               Vagrant::Util::Subprocess.execute("grep", "nfsd", "/proc/filesystems").exit_code == 0
           end
         end
@@ -173,15 +177,14 @@ module VagrantPlugins
         def self.nfs_write_exports(new_exports_content)
           if(nfs_exports_content != new_exports_content.strip)
             begin
+              exports_path = Pathname.new(NFS_EXPORTS_PATH)
+
               # Write contents out to temporary file
-              new_exports_file = Tempfile.create('vagrant')
+              new_exports_path = File.join(Dir.tmpdir, "vagrant-exports")
+              FileUtils.rm_f(new_exports_path)
+              new_exports_file = File.open(new_exports_path, "w+")
               new_exports_file.puts(new_exports_content)
               new_exports_file.close
-              new_exports_path = new_exports_file.path
-
-              # Only use "sudo" if we can't write to /etc/exports directly
-              sudo_command = ""
-              sudo_command = "sudo " if !File.writable?(NFS_EXPORTS_PATH)
 
               # Ensure new file mode and uid/gid match existing file to replace
               existing_stat = File.stat(NFS_EXPORTS_PATH)
@@ -190,7 +193,7 @@ module VagrantPlugins
                 File.chmod(existing_stat.mode, new_exports_path)
               end
               if existing_stat.uid != new_stat.uid || existing_stat.gid != new_stat.gid
-                chown_cmd = "#{sudo_command}chown #{existing_stat.uid}:#{existing_stat.gid} #{new_exports_path}"
+                chown_cmd = "sudo chown #{existing_stat.uid}:#{existing_stat.gid} #{new_exports_path}"
                 result = Vagrant::Util::Subprocess.execute(*Shellwords.split(chown_cmd))
                 if result.exit_code != 0
                   raise Vagrant::Errors::NFSExportsFailed,
@@ -200,6 +203,7 @@ module VagrantPlugins
                 end
               end
               # Always force move the file to prevent overwrite prompting
+              sudo_command = "sudo " if !exports_path.writable? || !exports_path.dirname.writable?
               mv_cmd = "#{sudo_command}mv -f #{new_exports_path} #{NFS_EXPORTS_PATH}"
               result = Vagrant::Util::Subprocess.execute(*Shellwords.split(mv_cmd))
               if result.exit_code != 0
@@ -209,7 +213,7 @@ module VagrantPlugins
                   stdout: result.stdout
               end
             ensure
-              if File.exist?(new_exports_path)
+              if !new_exports_path.nil? && File.exist?(new_exports_path)
                 File.unlink(new_exports_path)
               end
             end
@@ -262,6 +266,24 @@ module VagrantPlugins
           Vagrant::Util::Subprocess.execute(*Shellwords.split(check_command)).exit_code == 0
         end
 
+        def self.modinfo_path
+          if !defined?(@_modinfo_path)
+            @_modinfo_path = Vagrant::Util::Which.which("modinfo")
+
+            if @_modinfo_path.to_s.empty?
+              path = "/sbin/modinfo"
+              if File.file?(path)
+                @_modinfo_path = path
+              end
+            end
+
+            if @_modinfo_path.to_s.empty?
+              @_modinfo_path = "modinfo"
+            end
+          end
+          @_modinfo_path
+        end
+
         # @private
         # Reset the cached values for capability. This is not considered a public
         # API and should only be used for testing.
