@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "vagrant/util/which"
 
 module VagrantPlugins
@@ -13,7 +16,11 @@ module VagrantPlugins
             elsif Vagrant::Util::Which.which("rdesktop")
               "rdesktop"
             else
-              raise Vagrant::Errors::LinuxRDPClientNotFound
+              if Vagrant::Util::Platform.wsl?
+                "mstsc.exe"
+              else
+                raise Vagrant::Errors::LinuxRDPClientNotFound
+              end
             end
 
           args = []
@@ -30,10 +37,21 @@ module VagrantPlugins
             args << "-p" << rdp_info[:password] if rdp_info[:password]
             args += rdp_info[:extra_args] if rdp_info[:extra_args]
             args << "#{rdp_info[:host]}:#{rdp_info[:port]}"
+          when "mstsc.exe"
+            # Setup password
+            cmdKeyArgs = [
+              "/add:#{rdp_info[:host]}:#{rdp_info[:port]}",
+              "/user:#{rdp_info[:username]}",
+              "/pass:#{rdp_info[:password]}",
+            ]
+            Vagrant::Util::Subprocess.execute("cmdkey.exe", *cmdKeyArgs)
+
+            args = ["/v:#{rdp_info[:host]}:#{rdp_info[:port]}"]
+            args += rdp_info[:extra_args] if rdp_info[:extra_args]
           end
 
           # Finally, run the client.
-          Vagrant::Util::Subprocess.execute(rdp_client, *args)
+          Vagrant::Util::Subprocess.execute(rdp_client, *args, {:detach => true})
         end
       end
     end
