@@ -1,9 +1,12 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 module VagrantPlugins
   module HostSlackware
     module Cap
       class NFS
         def self.nfs_check_command(env)
-          "pidof nfsd >/dev/null"
+          "/sbin/pidof nfsd >/dev/null"
         end
 
         def self.nfs_start_command(env)
