@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 module VagrantPlugins
   module DockerProvider
     module Action
@@ -7,7 +10,7 @@ module VagrantPlugins
         end
 
         def call(env)
-          env[:result] = !!env[:machine].provider_config.build_dir
+          env[:result] = (!!env[:machine].provider_config.build_dir || !!env[:machine].provider_config.git_repo)
           @app.call(env)
         end
       end
