@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "vagrant/util/busy"
 require "vagrant/util/subprocess"
 
@@ -27,7 +30,17 @@ module VagrantPlugins
               stdout: result.stdout
           end
 
-          result.stdout
+          if opts
+            if opts[:with_stderr]
+              return result.stdout + " " + result.stderr
+            else
+              return result.stdout
+            end
+          end
+        end
+
+        def windows?
+          ::Vagrant::Util::Platform.windows? || ::Vagrant::Util::Platform.wsl?
         end
       end
     end
