@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 module VagrantPlugins
   module ProviderVirtualBox
     module Action
@@ -30,8 +33,8 @@ module VagrantPlugins
                   processed_command + [retryable: true])
               rescue Vagrant::Errors::VBoxManageError => e
                 raise Vagrant::Errors::VMCustomizationFailed, {
-                  :command => command,
-                  :error   => e.inspect
+                  command: command,
+                  error:   e.inspect
                 }
               end
             end
