@@ -1,4 +1,8 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "fileutils"
+require 'vagrant/util/platform'
 
 module VagrantPlugins
   module ProviderVirtualBox
@@ -22,8 +26,10 @@ module VagrantPlugins
         def export
           @env[:ui].info I18n.t("vagrant.actions.vm.export.exporting")
           @env[:machine].provider.driver.export(ovf_path) do |progress|
-            @env[:ui].clear_line
-            @env[:ui].report_progress(progress.percent, 100, false)
+            @env[:ui].rewriting do |ui|
+              ui.clear_line
+              ui.report_progress(progress.percent, 100, false)
+            end
           end
 
           # Clear the line a final time so the next data can appear
@@ -32,7 +38,15 @@ module VagrantPlugins
         end
 
         def ovf_path
-          File.join(@env["export.temp_dir"], "box.ovf")
+          path = File.join(@env["export.temp_dir"], "box.ovf")
+
+          # If we're within WSL, we should use the correct path rather than
+          # the mnt path. GH-9059
+          if Vagrant::Util::Platform.wsl?
+            path = Vagrant::Util::Platform.wsl_to_windows_path(path)
+          end
+
+          return path
         end
       end
     end
