@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 module VagrantPlugins
   module ProviderVirtualBox
     module Action
@@ -7,7 +10,7 @@ module VagrantPlugins
         end
 
         def call(env)
-          if env[:machine].provider.state.id == :running
+          if env[:machine].state.id == :running
             env[:ui].info I18n.t("vagrant.actions.vm.suspend.suspending")
             env[:machine].provider.driver.suspend
           end
