@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require_relative "base"
 
 module VagrantPlugins
@@ -5,16 +8,25 @@ module VagrantPlugins
     module Config
       class Host < Base
 
-        attr_accessor :ask_sudo_pass
+        attr_accessor :ask_become_pass
         attr_accessor :ask_vault_pass
         attr_accessor :force_remote_user
         attr_accessor :host_key_checking
         attr_accessor :raw_ssh_args
 
+        #
+        # Deprecated options
+        #
+        alias :ask_sudo_pass :ask_become_pass
+        def ask_sudo_pass=(value)
+          show_deprecation_info 'ask_sudo_pass', 'ask_become_pass'
+          @ask_become_pass = value
+        end
+
         def initialize
           super
 
-          @ask_sudo_pass       = false
+          @ask_become_pass     = false
           @ask_vault_pass      = false
           @force_remote_user   = true
           @host_key_checking   = false
@@ -24,7 +36,7 @@ module VagrantPlugins
         def finalize!
           super
 
-          @ask_sudo_pass       = false if @ask_sudo_pass     != true
+          @ask_become_pass     = false if @ask_become_pass   != true
           @ask_vault_pass      = false if @ask_vault_pass    != true
           @force_remote_user   = true  if @force_remote_user != false
           @host_key_checking   = false if @host_key_checking != true
