@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "vagrant"
 
 module VagrantPlugins
@@ -35,7 +38,7 @@ module VagrantPlugins
 
       guest_capability("suse", "cfengine_install") do
         require_relative "cap/suse/cfengine_install"
-        Cap::SuSE::CFEngineInstall
+        Cap::SUSE::CFEngineInstall
       end
 
       provisioner(:cfengine) do
