@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 module VagrantPlugins
   module Chef
     module Cap
@@ -6,11 +9,13 @@ module VagrantPlugins
           # Check if Chef is installed at the given version.
           # @return [true, false]
           def self.chef_installed(machine, product, version)
-            knife = "/opt/#{product}/bin/knife"
-            command = "test -x #{knife}"
+            product_name = product == 'chef-workstation' ? 'chef-workstation' : 'chef'
+            verify_bin = product_name == 'chef-workstation' ? 'chef' : 'chef-client'
+            verify_path = "/opt/#{product_name}/bin/#{verify_bin}"
+            command = "test -x #{verify_path}"
 
             if version != :latest
-              command << "&& #{knife} --version | grep 'Chef: #{version}'"
+              command << "&& #{verify_path} --version | grep '#{version}'"
             end
 
             machine.communicate.test(command, sudo: true)
