@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 module VagrantPlugins
   module Chef
     module Cap
@@ -6,10 +9,11 @@ module VagrantPlugins
           # Check if Chef is installed at the given version.
           # @return [true, false]
           def self.chef_installed(machine, product, version)
+            verify_bin = product == 'chef-workstation' ? 'chef' : 'chef-client'
             if version != :latest
-              command = 'if ((&knife --version) -Match "Chef: ' + version.to_s + '"){ exit 0 } else { exit 1 }'
+              command = 'if ((&' + verify_bin + ' --version) -Match "' + version.to_s + '"){ exit 0 } else { exit 1 }'
             else
-              command = 'if ((&knife --version) -Match "Chef: *"){ exit 0 } else { exit 1 }'
+              command = 'if ((&' + verify_bin + ' --version) -Match "Chef*"){ exit 0 } else { exit 1 }'
             end
             machine.communicate.test(command, sudo: true)
           end
