@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require 'pathname'
 
 require 'vagrant'
@@ -73,35 +76,42 @@ module VagrantPlugins
             @machine.ui.warn(I18n.t("vagrant.chef_run_list_empty"))
           end
 
-          if @machine.guest.capability?(:wait_for_reboot)
-            @machine.guest.capability(:wait_for_reboot)
-          end
-
           command = CommandBuilder.command(:client, @config,
             windows: windows?,
             colored: @machine.env.ui.color?,
           )
-
+          
+          still_active = 259 #provisioner has asked chef to reboot 
+          
           @config.attempts.times do |attempt|
-            if attempt == 0
-              @machine.ui.info I18n.t("vagrant.provisioners.chef.running_client")
-            else
-              @machine.ui.info I18n.t("vagrant.provisioners.chef.running_client_again")
+            exit_status = 0
+            while exit_status == 0 || exit_status == still_active 
+              if @machine.guest.capability?(:wait_for_reboot)
+                @machine.guest.capability(:wait_for_reboot)
+              elsif attempt > 0
+                sleep 10
+                @machine.communicate.wait_for_ready(@machine.config.vm.boot_timeout)
+              end
+              if attempt == 0
+                @machine.ui.info I18n.t("vagrant.provisioners.chef.running_client")
+              else
+                @machine.ui.info I18n.t("vagrant.provisioners.chef.running_client_again")
+              end
+
+              opts = { error_check: false, elevated: true }
+              exit_status = @machine.communicate.sudo(command, opts) do |type, data|
+                # Output the data with the proper color based on the stream.
+                color = type == :stdout ? :green : :red
+
+                data = data.chomp
+                next if data.empty?
+
+                @machine.ui.info(data, color: color)
+              end
+
+              # There is no need to run Chef again if it converges
+              return if exit_status == 0
             end
-
-            opts = { error_check: false, elevated: true }
-            exit_status = @machine.communicate.sudo(command, opts) do |type, data|
-              # Output the data with the proper color based on the stream.
-              color = type == :stdout ? :green : :red
-
-              data = data.chomp
-              next if data.empty?
-
-              @machine.ui.info(data, color: color)
-            end
-
-            # There is no need to run Chef again if it converges
-            return if exit_status == 0
           end
 
           # If we reached this point then Chef never converged! Error.
