@@ -1,4 +1,8 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require 'json'
+require_relative "bootstrap_downloader"
 
 module VagrantPlugins
   module Salt
@@ -44,16 +48,17 @@ module VagrantPlugins
       end
 
       def binaries_found
+        if @machine.config.vm.communicator == :winrm
+          which_cmd = "Get-Command"
+        else
+          which_cmd = "which"
+        end
+
         ## Determine States, ie: install vs configure
         desired_binaries = []
         if !@config.no_minion
-          if @machine.config.vm.communicator == :winrm
-            desired_binaries.push('C:\\salt\\salt-minion.bat')
-            desired_binaries.push('C:\\salt\\salt-call.bat')
-          else
-            desired_binaries.push('salt-minion')
-            desired_binaries.push('salt-call')
-          end
+          desired_binaries.push('salt-minion')
+          desired_binaries.push('salt-call')
         end
 
         if @config.install_master
@@ -77,7 +82,7 @@ module VagrantPlugins
         found = true
         for binary in desired_binaries
           @machine.env.ui.info "Checking if %s is installed" % binary
-          if !@machine.communicate.test("which %s" % binary)
+          if !@machine.communicate.test("%s %s" % [which_cmd, binary])
             @machine.env.ui.info "%s was not found." % binary
             found = false
           else
@@ -89,7 +94,7 @@ module VagrantPlugins
       end
 
       def need_configure
-        @config.minion_config or @config.minion_key or @config.master_config or @config.master_key or @config.grains_config or @config.version
+        @config.minion_config or @config.minion_key or @config.master_config or @config.master_key or @config.grains_config or @config.version or @config.minion_json_config or @config.master_json_config
       end
 
       def need_install
@@ -110,11 +115,21 @@ module VagrantPlugins
 
       # Generates option string for bootstrap script
       def bootstrap_options(install, configure, config_dir)
-        options = ""
-
         # Any extra options passed to bootstrap
         if @config.bootstrap_options
-          options = "%s %s" % [options, @config.bootstrap_options]
+          options = @config.bootstrap_options
+        else
+          options = ""
+        end
+
+        if @config.master_json_config && @machine.config.vm.communicator != :winrm
+          config = @config.master_json_config
+          options = "%s -J '#{config}'" % [options]
+        end
+
+        if @config.minion_json_config && @machine.config.vm.communicator != :winrm
+          config = @config.minion_json_config
+          options = "%s -j '#{config}'" % [options]
         end
 
         if configure && @machine.config.vm.communicator != :winrm
@@ -132,8 +147,12 @@ module VagrantPlugins
           options = "#{options} -k #{seed_dir}"
         end
 
-        if configure && !install && @machine.config.vm.communicator != :winrm
-          options = "%s -C" % options
+        if configure && !install
+          if @machine.config.vm.communicator == :winrm
+            options = "%s -ConfigureOnly" % options
+          else
+            options = "%s -C" % options
+          end
         end
 
         if @config.install_master && @machine.config.vm.communicator != :winrm
@@ -148,6 +167,10 @@ module VagrantPlugins
           options = "%s -N" % options
         end
 
+        if @config.python_version && @machine.config.vm.communicator != :winrm
+          options = "%s -x python%s" % [options, @config.python_version]
+        end
+
         if @config.install_type && @machine.config.vm.communicator != :winrm
           options = "%s %s" % [options, @config.install_type]
         end
@@ -166,7 +189,19 @@ module VagrantPlugins
       ## Actions
       # Get pillar string to pass with the salt command
       def get_pillar
-        " pillar='#{@config.pillar_data.to_json}'" if !@config.pillar_data.empty?
+        if !@config.pillar_data.empty?
+          if @machine.config.vm.communicator == :winrm
+            # ' doesn't have any special behavior on the command prompt,
+            # so '{"x":"y"}' becomes '{x:y}' with literal single quotes.
+            # However, """ will become " , and \\""" will become \" .
+            # Use \\"" instead of \\""" for literal inner-value quotes
+            # to avoid issue with odd number of quotes.
+            # --% disables special PowerShell parsing on the rest of the line.
+            " --% pillar=#{@config.pillar_data.to_json.gsub(/(?<!\\)\"/, '"""').gsub(/\\\"/, %q(\\\\\""))}"
+          else
+            " pillar='#{@config.pillar_data.to_json}'"
+          end
+        end
       end
 
       # Get colorization option string to pass with the salt command
@@ -248,11 +283,9 @@ module VagrantPlugins
         if @config.bootstrap_script
           bootstrap_abs_path = expanded_path(@config.bootstrap_script)
         else
-          if @machine.config.vm.communicator == :winrm
-            bootstrap_abs_path = Pathname.new("../bootstrap-salt.ps1").expand_path(__FILE__)
-          else
-            bootstrap_abs_path = Pathname.new("../bootstrap-salt.sh").expand_path(__FILE__)
-          end
+          bootstrap_downloader = BootstrapDownloader.new(@machine.config.vm.guest)
+          bootstrap_script = bootstrap_downloader.get_bootstrap_script
+          bootstrap_abs_path = expanded_path(bootstrap_script.path)
         end
 
         return bootstrap_abs_path
@@ -294,6 +327,9 @@ module VagrantPlugins
             end
             bootstrap_destination = File.join(config_dir, "bootstrap_salt.ps1")
           else
+            if @config.version
+              options += " %s" % @config.version
+            end
             bootstrap_destination = File.join(config_dir, "bootstrap_salt.sh")
           end
 
@@ -375,7 +411,7 @@ module VagrantPlugins
 
           @machine.env.ui.info "Calling state.highstate... (this may take a while)"
           if @config.install_master
-            unless @config.masterless?
+            unless @config.masterless
               @machine.communicate.sudo("salt '*' saltutil.sync_all")
             end
             options = "#{get_masterless}#{get_loglevel}#{get_colorize}#{get_pillar}#{get_salt_args}"
@@ -387,18 +423,18 @@ module VagrantPlugins
           else
             if @machine.config.vm.communicator == :winrm
               opts = { elevated: true }
-              unless @config.masterless?
-                @machine.communicate.execute("C:\\salt\\salt-call.bat saltutil.sync_all", opts)
+              unless @config.masterless
+                @machine.communicate.execute("salt-call saltutil.sync_all", opts)
               end
               # TODO: something equivalent to { error_key: :ssh_bad_exit_status_muted }?
               options = "#{get_masterless}#{get_loglevel}#{get_colorize}#{get_pillar}#{get_call_args}"
-              @machine.communicate.execute("C:\\salt\\salt-call.bat state.highstate --retcode-passthrough#{options}", opts) do |type, data|
+              @machine.communicate.execute("salt-call state.highstate --retcode-passthrough#{options}", opts) do |type, data|
                 if @config.verbose
                   @machine.env.ui.info(data.rstrip)
                 end
               end
             else
-              unless @config.masterless?
+              unless @config.masterless
                 @machine.communicate.sudo("salt-call saltutil.sync_all")
               end
               options = "#{get_masterless}#{get_loglevel}#{get_colorize}#{get_pillar}#{get_call_args}"
