@@ -1,7 +1,11 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "net/ftp"
 require "pathname"
 
 require_relative "adapter"
+require_relative "errors"
 
 module VagrantPlugins
   module FTPPush
@@ -17,19 +21,24 @@ module VagrantPlugins
       def push
         # Grab files early so if there's an exception or issue, we don't have to
         # wait and close the (S)FTP connection as well
-        files = Hash[*all_files.flat_map do |file|
-          relative_path = relative_path_for(file, config.dir)
-          destination = File.join(config.destination, relative_path)
-          file = File.expand_path(file, env.root_path)
-          [file, destination]
-        end]
+        files = nil
+        begin
+          files = Hash[*all_files.flat_map do |file|
+              relative_path = relative_path_for(file, base_dir)
+              destination = File.join(config.destination, relative_path)
+              file = File.expand_path(file, env.root_path)
+              [file, destination]
+            end]
+        rescue SystemStackError
+          raise Errors::TooManyFiles
+        end
 
         ftp = "#{config.username}@#{config.host}:#{config.destination}"
-        env.ui.info "Uploading #{env.root_path} to #{ftp}"
+        env.ui.info "Uploading #{files.length} files to #{env.root_path} to #{ftp}"
 
         connect do |ftp|
           files.each do |local, remote|
-            @logger.info "Uploading #{local} => #{remote}"
+            env.ui.info "Uploading #{local} => #{remote}"
             ftp.upload(local, remote)
           end
         end
@@ -48,7 +57,7 @@ module VagrantPlugins
       # only returns **files**, not folders or symlinks!
       # @return [Array<String>]
       def all_files
-        files = glob("#{config.dir}/**/*") + includes_files
+        files = glob("#{base_dir}/**/*") + includes_files
         filter_excludes!(files, config.excludes)
         files.reject! { |f| !File.file?(f) }
         files
@@ -58,7 +67,7 @@ module VagrantPlugins
       # @return [Array<String>]
       def includes_files
         includes = config.includes.flat_map do |i|
-          path = absolute_path_for(i, config.dir)
+          path = absolute_path_for(i, base_dir)
           [path, "#{path}/**/*"]
         end
 
@@ -115,6 +124,14 @@ module VagrantPlugins
       rescue ArgumentError
         return path
       end
+
+      def base_dir
+        if Pathname.new(config.dir).absolute?
+          config.dir
+        else
+          File.join(File.expand_path(env.root_path), config.dir)
+        end
+      end
     end
   end
 end
