@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require 'optparse'
 
 require "vagrant/action/builtin/mixin_synced_folders"
@@ -15,6 +18,7 @@ module VagrantPlugins
         end
 
         def execute
+          options = {}
           opts = OptionParser.new do |o|
             o.banner = "Usage: vagrant rsync [vm-name]"
             o.separator ""
@@ -23,6 +27,9 @@ module VagrantPlugins
             o.separator ""
             o.separator "Options:"
             o.separator ""
+            o.on("--[no-]rsync-chown", "Use rsync to modify ownership") do |chown|
+              options[:rsync_chown] = chown
+            end
           end
 
           # Parse the options and return if we don't have any target.
@@ -59,6 +66,9 @@ module VagrantPlugins
 
             # Sync them!
             folders.each do |id, folder_opts|
+              if options.has_key?(:rsync_chown)
+                folder_opts = folder_opts.merge(rsync_ownership: options[:rsync_chown])
+              end
               RsyncHelper.rsync_single(machine, ssh_info, folder_opts)
             end
           end
