@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "log4r"
 
 require "vagrant/util/subprocess"
@@ -48,6 +51,12 @@ module VagrantPlugins
           RsyncHelper.rsync_single(machine, ssh_info, folder_opts)
         end
       end
+
+      # Enable rsync synced folders within WSL when in use
+      # on non-DrvFs file systems
+      def self.wsl_allow_non_drvfs?
+        true
+      end
     end
   end
 end
