@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "digest/md5"
 require "json"
 
@@ -39,26 +42,34 @@ module VagrantPlugins
         # If we need auth information, then ask the user.
         have_auth = false
         folders.each do |id, data|
-          if data[:smb_username] && data[:smb_password]
-            smb_username = data[:smb_username]
-            smb_password = data[:smb_password]
+          smb_username = data[:smb_username] if data[:smb_username]
+          smb_password = data[:smb_password] if data[:smb_password]
+          if smb_username && smb_password
             have_auth = true
             break
           end
         end
 
+        modify_username = false
         if !have_auth
           machine.ui.detail(I18n.t("vagrant_sf_smb.warning_password") + "\n ")
           retries = 0
           while retries < CREDENTIAL_RETRY_MAX do
-            smb_username = machine.ui.ask("Username: ")
+            if smb_username
+              username = machine.ui.ask("Username (#{smb_username}): ")
+              smb_username = username if username != ""
+              modify_username = true
+            else
+              smb_username = machine.ui.ask("Username (user[@domain]): ")
+            end
+
             smb_password = machine.ui.ask("Password (will be hidden): ", echo: false)
             auth_success = true
 
             if machine.env.host.capability?(:smb_validate_password)
               Vagrant::Util::CredentialScrubber.sensitive(smb_password)
               auth_success = machine.env.host.capability(:smb_validate_password,
-                smb_username, smb_password)
+                machine, smb_username, smb_password)
             end
 
             break if auth_success
@@ -77,7 +88,12 @@ module VagrantPlugins
         end
 
         folders.each do |id, data|
-          data[:smb_username] ||= smb_username
+          if modify_username
+            # Only override original username if user requests to
+            data[:smb_username] = smb_username
+          else
+            data[:smb_username] ||= smb_username
+          end
           data[:smb_password] ||= smb_password
 
           # Register password as sensitive
@@ -139,6 +155,15 @@ module VagrantPlugins
             guest: data[:guestpath].to_s))
           machine.guest.capability(
             :mount_smb_shared_folder, data[:smb_id], data[:guestpath], data)
+
+          clean_folder_configuration(data)
+        end
+      end
+
+      # Nothing to do here but ensure folder options are scrubbed
+      def disable(machine, folders, opts)
+        folders.each do |_, data|
+          clean_folder_configuration(data)
         end
       end
 
@@ -147,6 +172,18 @@ module VagrantPlugins
           machine.env.host.capability(:smb_cleanup, machine, opts)
         end
       end
+
+      protected
+
+      # Remove data that should not be persisted within folder
+      # specific configuration
+      #
+      # @param [Hash] data Folder configuration
+      def clean_folder_configuration(data)
+        return if !data.is_a?(Hash)
+        data.delete(:smb_password)
+        nil
+      end
     end
   end
 end
