@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require_relative "../../../base"
 require_relative "../../../../../plugins/commands/init/command"
 
@@ -33,7 +36,6 @@ describe VagrantPlugins::CommandInit::Command do
     it "creates a minimal Vagrantfile" do
       described_class.new(["-m"], env).execute
       contents = File.read(vagrantfile_path)
-      expect(contents).to_not match(/#/)
       expect(contents).to_not match(/provision/)
     end
 
@@ -55,6 +57,22 @@ describe VagrantPlugins::CommandInit::Command do
       expect(contents).to match(/config.vm.hostname = "vagrant.dev"/)
     end
 
+    it "creates a custom Vagrant file using a template provided from the environment" do
+      with_temp_env("VAGRANT_DEFAULT_TEMPLATE" => "test/unit/templates/commands/init/Vagrantfile") do
+        described_class.new([], env).execute
+      end
+      contents = File.read(vagrantfile_path)
+      expect(contents).to match(/config.vm.hostname = "vagrant.dev"/)
+    end
+
+    it "ignores the environmentally-set default template when a template is explicitly set" do
+      with_temp_env("VAGRANT_DEFAULT_TEMPLATE" => "/this_file_does_not_exist") do
+        described_class.new(["--template", "test/unit/templates/commands/init/Vagrantfile"], env).execute
+      end
+      contents = File.read(vagrantfile_path)
+      expect(contents).to match(/config.vm.hostname = "vagrant.dev"/)
+    end
+
     it "ignores the -m option when using a provided template" do
       described_class.new(["-m", "--template", ::Vagrant.source_root.join("test/unit/templates/commands/init/Vagrantfile").to_s], env).execute
       contents = File.read(vagrantfile_path)
@@ -95,10 +113,10 @@ describe VagrantPlugins::CommandInit::Command do
     end
 
     it "creates a Vagrantfile with a box and box_url" do
-      described_class.new(["hashicorp/precise64", "http://mybox.com"], env).execute
+      described_class.new(["hashicorp/precise64", "http://example.com"], env).execute
       contents = File.read(vagrantfile_path)
       expect(contents).to match(/config.vm.box = "hashicorp\/precise64"/)
-      expect(contents).to match(/config.vm.box_url = "http:\/\/mybox.com"/)
+      expect(contents).to match(/config.vm.box_url = "http:\/\/example.com"/)
     end
 
     it "creates a Vagrantfile with a box and box version" do
