@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require File.expand_path("../../../../../base", __FILE__)
 
 describe VagrantPlugins::CommandPlugin::Action::UninstallPlugin do
@@ -15,7 +18,7 @@ describe VagrantPlugins::CommandPlugin::Action::UninstallPlugin do
   end
 
   it "uninstalls the specified plugin" do
-    expect(manager).to receive(:uninstall_plugin).with("bar").ordered
+    expect(manager).to receive(:uninstall_plugin).with("bar", any_args).ordered
     expect(app).to receive(:call).ordered
 
     env[:plugin_name] = "bar"
