@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require File.expand_path("../../../../../base", __FILE__)
 
 describe VagrantPlugins::CommandPlugin::Action::UpdateGems do
@@ -11,8 +14,8 @@ describe VagrantPlugins::CommandPlugin::Action::UpdateGems do
   subject { described_class.new(app, env) }
 
   before do
-    Vagrant::Plugin::Manager.stub(instance: manager)
-    manager.stub(installed_specs: [])
+    allow(Vagrant::Plugin::Manager).to receive(:instance).and_return(manager)
+    allow(manager).to receive(:installed_specs).and_return([])
   end
 
   describe "#call" do
