@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require File.expand_path("../../../../base", __FILE__)
 
 require Vagrant.source_root.join("plugins/communicators/winrm/shell")
@@ -33,13 +36,17 @@ describe VagrantPlugins::CommunicatorWinRM::WinRMShell do
 
   describe "#upload" do
     let(:fm) { double("file_manager") }
+
+    before do
+      allow(WinRM::FS::FileManager).to receive(:new).with(connection)
+        .and_return(fm)
+    end
+
     it "should call file_manager.upload for each passed in path" do
       from = ["/path", "/path/folder", "/path/folder/file.py"]
       to = "/destination"
       size = 80
 
-      allow(WinRM::FS::FileManager).to receive(:new).with(connection)
-        .and_return(fm)
       allow(fm).to receive(:upload).and_return(size)
 
       expect(fm).to receive(:upload).exactly(from.size).times
@@ -51,13 +58,34 @@ describe VagrantPlugins::CommunicatorWinRM::WinRMShell do
       to = "/destination"
       size = 80
 
-      allow(WinRM::FS::FileManager).to receive(:new).with(connection)
-        .and_return(fm)
       allow(fm).to receive(:upload).and_return(size)
 
       expect(fm).to receive(:upload).exactly(1).times
       expect(subject.upload(from, to)).to eq(size)
     end
+
+    context "when source is a directory" do
+      let(:source) { "path/sourcedir" }
+
+      before do
+        allow(File).to receive(:directory?).with(/#{Regexp.escape(source)}/).and_return(true)
+      end
+
+      it "should add source directory name to destination" do
+        expect(fm).to receive(:upload) do |from, to|
+          expect(to).to include("sourcedir")
+        end
+        subject.upload(source, "/dest")
+      end
+
+      it "should not add source directory name to destination when source ends with '.'" do
+        source << "/."
+        expect(fm).to receive(:upload) do |from, to|
+          expect(to).to eq("/dest")
+        end
+        subject.upload(source, "/dest")
+      end
+    end
   end
 
   describe ".powershell" do
@@ -75,6 +103,20 @@ describe VagrantPlugins::CommunicatorWinRM::WinRMShell do
   end
 
   describe ".elevated" do
+    let(:eusername) { double("elevatedusername") }
+    let(:username) { double("username") }
+    let(:failed_output) { WinRM::Output.new.tap { |out|
+        out.exitcode = described_class.const_get(:INVALID_USERID_EXITCODE)
+        out << {stderr: "(10,8):UserId:"}
+        out << {stderr: "At line:72 char:1"}
+      } }
+
+    before do
+      allow(subject).to receive(:elevated_username).and_return(eusername)
+      allow(shell).to receive(:username).and_return(username)
+      allow(shell).to receive(:username=)
+    end
+
     it "should call winrm elevated" do
       expect(shell).to receive(:run).with("dir").and_return(output)
       expect(shell).to receive(:interactive_logon=).with(false)
@@ -93,6 +135,21 @@ describe VagrantPlugins::CommunicatorWinRM::WinRMShell do
       expect { subject.powershell("dir") }.to raise_error(
         VagrantPlugins::CommunicatorWinRM::Errors::ExecutionError)
     end
+
+    it "should use elevated username and retry on username failure" do
+      expect(subject).to receive(:elevated_username).and_return(eusername)
+      expect(shell).to receive(:run).with("dir").and_return(failed_output)
+      expect(shell).to receive(:run).with("dir").and_return(output)
+      expect(shell).to receive(:interactive_logon=).with(false)
+      expect(subject.elevated("dir").exitcode).to eq(0)
+    end
+
+    it "should not retry on username failure if elevated username is the same" do
+      expect(subject).to receive(:elevated_username).and_return(username)
+      expect(shell).to receive(:run).with("dir").and_return(failed_output)
+      expect(shell).to receive(:interactive_logon=).with(false)
+      expect(subject.elevated("dir").exitcode).to eq(failed_output.exitcode)
+    end
   end
 
   describe ".cmd" do
@@ -129,7 +186,7 @@ describe VagrantPlugins::CommunicatorWinRM::WinRMShell do
         # Note: The initialize for WinRMAuthorizationError may require a status_code as
         # the second argument in a future WinRM release. Currently it doesn't track the
         # status code.
-        WinRM::WinRMAuthorizationError.new("Oh no!! Unauthrorized")
+        WinRM::WinRMAuthorizationError.new("Oh no!! Unauthorized")
       )
       expect { subject.wql("select * from Win32_OperatingSystem") }.to raise_error(
         VagrantPlugins::CommunicatorWinRM::Errors::AuthenticationFailed)
@@ -178,4 +235,41 @@ describe VagrantPlugins::CommunicatorWinRM::WinRMShell do
     end
   end
 
+  describe "#elevated_username" do
+    let(:username) { "username" }
+
+    before do
+      allow(subject).to receive(:username).and_return(username)
+      allow(subject).to receive(:powershell)
+    end
+
+    it "should return username" do
+      expect(subject.send(:elevated_username)).to eq(username)
+    end
+
+    it "should attempt to get computer name" do
+      expect(subject).to receive(:powershell).with(/computername/)
+      subject.send(:elevated_username)
+    end
+
+    it "should prepend computer name when available" do
+      expect(subject).to receive(:powershell).with(/computername/).and_yield(:stdout, "COMPUTERNAME")
+      expect(subject.send(:elevated_username)).to eq("COMPUTERNAME\\#{username}")
+    end
+
+    it "should compute elevated username every time" do
+      expect(subject).to receive(:powershell).twice.with(/computername/).and_yield(:stdout, "COMPUTERNAME")
+      expect(subject.send(:elevated_username)).to eq("COMPUTERNAME\\#{username}")
+      expect(subject.send(:elevated_username)).to eq("COMPUTERNAME\\#{username}")
+    end
+
+    context "when username includes computer/domain name" do
+      let(:username) { "machine\\username" }
+
+      it "should not attempt to get computer name" do
+        expect(subject).not_to receive(:powershell)
+        expect(subject.send(:elevated_username)).to eq(username)
+      end
+    end
+  end
 end
