@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require_relative "../../../../base"
 
 describe "VagrantPlugins::GuestBSD::Cap::Halt" do
@@ -9,6 +12,7 @@ describe "VagrantPlugins::GuestBSD::Cap::Halt" do
 
   let(:machine) { double("machine") }
   let(:comm) { VagrantTests::DummyCommunicator::Communicator.new(machine) }
+  let(:shutdown_command) { "/sbin/shutdown -p now" }
 
   before do
     allow(machine).to receive(:communicate).and_return(comm)
@@ -22,12 +26,19 @@ describe "VagrantPlugins::GuestBSD::Cap::Halt" do
     let(:cap) { caps.get(:halt) }
 
     it "runs the shutdown command" do
-      comm.expect_command("/sbin/shutdown -p now")
+      comm.expect_command(shutdown_command)
       cap.halt(machine)
     end
 
     it "ignores an IOError" do
-      comm.stub_command("/sbin/shutdown -p now", raise: IOError)
+      comm.stub_command(shutdown_command, raise: IOError)
+      expect {
+        cap.halt(machine)
+      }.to_not raise_error
+    end
+
+    it "ignores a Vagrant::Errors::SSHDisconnected" do
+      comm.stub_command(shutdown_command, raise: Vagrant::Errors::SSHDisconnected)
       expect {
         cap.halt(machine)
       }.to_not raise_error
