@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require_relative "../../../../base"
 
 describe "VagrantPlugins::GuestOmniOS::Cap:RSync" do
@@ -40,8 +43,6 @@ describe "VagrantPlugins::GuestOmniOS::Cap:RSync" do
       expect(comm.received_commands[0]).to match(/'1.2.3.4:#{hostpath}' '#{guestpath}'/)
     end
 
-    it "mounts with options" do
-      pending "not yet implemented"
-    end
+    it "mounts with options"
   end
 end
