@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require_relative "../../../../base"
 
 describe "VagrantPlugins::VagrantPlugins::Cap::ConfigureNetworks" do
@@ -7,8 +10,8 @@ describe "VagrantPlugins::VagrantPlugins::Cap::ConfigureNetworks" do
   let(:communicator) { VagrantTests::DummyCommunicator::Communicator.new(machine) }
 
   before do
-    machine.stub(:communicate).and_return(communicator)
-    machine.stub(:config).and_return(config)
+    allow(machine).to receive(:communicate).and_return(communicator)
+    allow(machine).to receive(:config).and_return(config)
   end
 
   after do
