@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require_relative "../../../../base"
 
 require Vagrant.source_root.join("plugins/guests/windows/cap/change_host_name")
@@ -6,7 +9,8 @@ describe "VagrantPlugins::GuestWindows::Cap::ChangeHostName" do
   let(:described_class) do
     VagrantPlugins::GuestWindows::Plugin.components.guest_capabilities[:windows].get(:change_host_name)
   end
-  let(:machine) { double("machine") }
+  let(:machine) { double("machine", guest: guest) }
+  let(:guest) { double("guest") }
   let(:communicator) { VagrantTests::DummyCommunicator::Communicator.new(machine) }
 
   before do
@@ -34,6 +38,8 @@ describe "VagrantPlugins::GuestWindows::Cap::ChangeHostName" do
         'if (!([System.Net.Dns]::GetHostName() -eq \'newhostname\')) { exit 0 } exit 1',
         exit_code: 0)
       communicator.stub_command(rename_script, exit_code: 0)
+      allow(machine.guest).to receive(:capability)
+      allow(machine.guest).to receive(:capability?)
       described_class.change_host_name_and_wait(machine, 'newhostname', 0)
     end
 
