@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require_relative "../../../../base"
 
 require Vagrant.source_root.join("plugins/guests/windows/cap/mount_shared_folder")
@@ -29,7 +32,7 @@ describe "VagrantPlugins::GuestWindows::Cap::MountSharedFolder" do
           /.+scripts\/mount_volume.ps1/, options: {
               mount_point: "guestpath",
               share_name: "name",
-              vm_provider_unc_path: "\\\\vboxsrv\\name",
+              vm_provider_unc_path: "\\\\vboxsvr\\name",
             })
         described_class.mount_virtualbox_shared_folder(machine, 'name', 'guestpath', {})
       end
@@ -39,7 +42,7 @@ describe "VagrantPlugins::GuestWindows::Cap::MountSharedFolder" do
           kind_of(String), options: {
               mount_point: kind_of(String),
               share_name: "invalid-windows_sharename",
-              vm_provider_unc_path: "\\\\vboxsrv\\invalid-windows_sharename",
+              vm_provider_unc_path: "\\\\vboxsvr\\invalid-windows_sharename",
             })
         described_class.mount_virtualbox_shared_folder(machine, "/invalid-windows/sharename", "guestpath", {})
       end
@@ -98,6 +101,7 @@ describe "VagrantPlugins::GuestWindows::Cap::MountSharedFolder" do
               share_name: "name",
               vm_provider_unc_path: "\\\\host\\name",
             })
+        expect(machine.communicate).to receive(:execute).with("cmdkey /add:host /user:user /pass:\"pass\"", {:shell=>:powershell, :elevated=>true})
         described_class.mount_smb_shared_folder(machine, 'name', 'guestpath', {:smb_username => "user", :smb_password => "pass", :smb_host => "host"})
       end
     end
