@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require File.expand_path("../../../../base", __FILE__)
 
 require Vagrant.source_root.join("plugins/kernel_v2/config/vm")
@@ -7,7 +10,8 @@ describe VagrantPlugins::Kernel_V2::VMConfig do
 
   subject { described_class.new }
 
-  let(:machine) { double("machine") }
+  let(:provider) { double("provider") }
+  let(:machine) { double("machine", provider: provider, provider_name: "provider", name: "default") }
 
   def assert_invalid
     errors = subject.validate(machine)
@@ -36,6 +40,11 @@ describe VagrantPlugins::Kernel_V2::VMConfig do
     allow(machine).to receive(:env).and_return(env)
     allow(machine).to receive(:provider_config).and_return(nil)
     allow(machine).to receive(:provider_options).and_return({})
+    allow(machine).to receive_message_chain(:synced_folders, :types).and_return( {} )
+    allow(provider).to receive(:capability?).with(:validate_disk_ext).and_return(true)
+    allow(provider).to receive(:capability).with(:validate_disk_ext, "vdi").and_return(true)
+    allow(provider).to receive(:capability?).with(:set_default_disk_ext).and_return(true)
+    allow(provider).to receive(:capability).with(:set_default_disk_ext).and_return("vdi")
 
     subject.box = "foo"
   end
@@ -45,6 +54,27 @@ describe VagrantPlugins::Kernel_V2::VMConfig do
     assert_valid
   end
 
+  it "validates disables_host_modification option" do
+    subject.allow_hosts_modification = true
+    subject.finalize!
+    assert_valid
+
+    subject.allow_hosts_modification = false
+    subject.finalize!
+    assert_valid
+
+    subject.allow_hosts_modification = "truthy"
+    subject.finalize!
+    assert_invalid
+  end
+
+  it "does not check for fstab caps if already set" do
+    expect(machine).to_not receive(:synced_folder_types)
+    subject.allow_fstab_modification = true
+    subject.finalize!
+    assert_valid
+  end
+
   describe "#base_mac" do
     it "defaults properly" do
       subject.finalize!
@@ -52,6 +82,13 @@ describe VagrantPlugins::Kernel_V2::VMConfig do
     end
   end
 
+  describe "#base_address" do
+    it "defaults properly" do
+      subject.finalize!
+      expect(subject.base_address).to be_nil
+    end
+  end
+
   describe "#box" do
     it "is required" do
       subject.box = nil
@@ -59,6 +96,12 @@ describe VagrantPlugins::Kernel_V2::VMConfig do
       assert_invalid
     end
 
+    it "cannot be an empty string" do
+      subject.box = ""
+      subject.finalize!
+      assert_invalid
+    end
+
     it "is not required if the provider says so" do
       machine.provider_options[:box_optional] = true
       subject.box = nil
@@ -73,6 +116,34 @@ describe VagrantPlugins::Kernel_V2::VMConfig do
     end
   end
 
+  describe "#box_architecture" do
+    it "is not required" do
+      subject.box_architecture = nil
+      subject.finalize!
+      assert_valid
+    end
+
+    it "is :auto by default" do
+      subject.finalize!
+      assert_valid
+      expect(subject.box_architecture).to eq(:auto)
+    end
+
+    it "can be set to custom value" do
+      subject.box_architecture = "test-arch"
+      subject.finalize!
+      assert_valid
+      expect(subject.box_architecture).to eq("test-arch")
+    end
+
+    it "is converted to string" do
+      subject.box_architecture = :test_arch
+      subject.finalize!
+      assert_valid
+      expect(subject.box_architecture).to eq("test_arch")
+    end
+  end
+
   context "#box_check_update" do
     it "defaults to true" do
       with_temp_env("VAGRANT_BOX_UPDATE_CHECK_DISABLE" => "") do
@@ -292,6 +363,32 @@ describe VagrantPlugins::Kernel_V2::VMConfig do
       subject.finalize!
       assert_invalid
     end
+
+    it "is an error if multiple networks set hostname" do
+      subject.network "public_network", ip: "192.168.0.1", hostname: true
+      subject.network "public_network", ip: "192.168.0.2", hostname: true
+      subject.finalize!
+      assert_invalid
+    end
+
+    it "is an error if networks set hostname without ip" do
+      subject.network "public_network", hostname: true
+      subject.finalize!
+      assert_invalid
+    end
+
+    it "is not an error if hostname non-bool" do
+      subject.network "public_network",  ip: "192.168.0.1", hostname: "true"
+      subject.finalize!
+      assert_valid
+    end
+
+    it "is not an error if one hostname is true" do
+      subject.network "public_network",  ip: "192.168.0.1", hostname: true
+      subject.network "public_network",  ip: "192.168.0.2", hostname: false
+      subject.finalize!
+      assert_valid
+    end
   end
 
   describe "#post_up_message" do
@@ -368,24 +465,41 @@ describe VagrantPlugins::Kernel_V2::VMConfig do
       expect { subject.finalize! }.
         to raise_error(Vagrant::Errors::VagrantfileLoadError)
     end
+
+    it "ignores providers entirely if flag is provided" do
+      subject.provider "virtualbox" do |vb|
+        vb.nope = true
+      end
+
+      subject.provider "virtualbox" do |vb|
+        vb.not_real = "foo"
+      end
+
+      subject.finalize!
+      errors = subject.validate(machine, true)
+      expect(errors).to eq({"vm"=>[]})
+    end
   end
 
   describe "#provision" do
     it "stores the provisioners" do
       subject.provision("shell", inline: "foo")
       subject.provision("shell", inline: "bar", run: "always") { |s| s.path = "baz" }
+      subject.provision("shell", inline: "foo", communicator_required: false)
       subject.finalize!
 
       r = subject.provisioners
-      expect(r.length).to eql(2)
+      expect(r.length).to eql(3)
       expect(r[0].run).to be_nil
       expect(r[0].config.inline).to eql("foo")
       expect(r[1].config.inline).to eql("bar")
       expect(r[1].config.path).to eql("baz")
       expect(r[1].run).to eql(:always)
+      expect(r[1].communicator_required).to eql(true)
+      expect(r[2].communicator_required).to eql(false)
     end
 
-    it "allows provisioner settings to be overriden" do
+    it "allows provisioner settings to be overridden" do
       subject.provision("s", path: "foo", type: "shell") { |s| s.inline = "foo" }
       subject.provision("s", inline: "bar", type: "shell") { |s| s.args = "bar" }
       subject.finalize!
@@ -528,6 +642,76 @@ describe VagrantPlugins::Kernel_V2::VMConfig do
     end
   end
 
+  describe "#disk" do
+    before(:each) do
+      allow(Vagrant::Util::Experimental).to receive(:feature_enabled?).
+        with("disks").and_return("true")
+    end
+
+    it "stores the disks" do
+      subject.disk(:disk, size: 100, primary: true)
+      subject.disk(:disk, size: 1000, name: "storage")
+      subject.finalize!
+
+      assert_valid
+
+      d = subject.disks
+      expect(d.length).to eql(2)
+      expect(d[0].size).to eql(100)
+      expect(d[1].size).to eql(1000)
+      expect(d[1].name).to eql("storage")
+    end
+
+    it "raises an error with duplicate names" do
+      subject.disk(:disk, size: 100, name: "foo")
+      subject.disk(:disk, size: 1000, name: "foo", primary: false)
+      subject.finalize!
+      assert_invalid
+    end
+
+    it "raises an error with duplicate disk files" do
+      allow(File).to receive(:file?).with("bar.vmdk").and_return(true)
+      subject.disk(:disk, size: 100, name: "foo1", file: "bar.vmdk")
+      subject.disk(:disk, size: 100, name: "foo2", file: "bar.vmdk")
+      subject.finalize!
+      assert_invalid
+    end
+
+    it "does not merge duplicate disks" do
+      subject.disk(:disk, size: 1000, primary: false, name: "storage")
+      subject.disk(:disk, size: 1000, primary: false, name: "backup")
+
+      merged = subject.merge(subject)
+      merged_disks = merged.disks
+
+      expect(merged_disks.length).to eql(2)
+    end
+
+    it "ignores non-overriding runs" do
+      subject.disk(:disk, name: "foo")
+
+      other = described_class.new
+      other.disk(:disk, name: "bar", primary: false)
+
+      merged = subject.merge(other)
+      merged_disks = merged.disks
+
+      expect(merged_disks.length).to eql(2)
+      expect(merged_disks[0].name).to eq("foo")
+      expect(merged_disks[1].name).to eq("bar")
+    end
+
+    it "adds provider config with `__` config form" do
+      subject.disk(:disk, size: 1000, primary: false, name: "storage",  provider__something: "special")
+      expect(subject.disks[0].provider_config).to eq({:provider=>{:something=>"special"}})
+    end
+
+    it "adds provider config with Hash config form" do
+      subject.disk(:disk, size: 1000, primary: false, name: "storage",  provider: {something: "special"})
+      expect(subject.disks[0].provider_config).to eq({:provider=>{:something=>"special"}})
+    end
+  end
+
   describe "#synced_folder(s)" do
     it "defaults to sharing the current directory" do
       subject.finalize!
@@ -557,8 +741,21 @@ describe VagrantPlugins::Kernel_V2::VMConfig do
       expect(sf["/vagrant"][:foo]).to eq(:bar)
     end
 
-    it "is not an error if guest path is empty" do
-      subject.synced_folder(".", "")
+    # This is a little bit of a special case since nfs can be specified
+    # as `type: "nfs"` or `nfs: true`
+    it "properly overrides nfs" do
+      subject.synced_folder(".", "/vagrant", nfs: true)
+      subject.synced_folder(".", "/vagrant", type: "rsync")
+      subject.finalize!
+      sf = subject.synced_folders
+      expect(sf.length).to eq(1)
+      expect(sf).to have_key("/vagrant")
+      expect(sf["/vagrant"][:type]).to be(:rsync)
+      expect(sf["/vagrant"][:nfs]).to eq(nil)
+    end
+
+    it "is not an error if guest path is empty but name is not" do
+      subject.synced_folder(".", "", name: "my-vagrant-folder")
       subject.finalize!
       assert_valid
     end
@@ -577,6 +774,91 @@ describe VagrantPlugins::Kernel_V2::VMConfig do
       expect(sf).to have_key("my-vagrant-folder")
       expect(sf["my-vagrant-folder"][:hostpath]).to eq(".")
     end
+
+    it "requires either guest path or name" do
+      subject.synced_folder(".", name: nil, guestpath: nil)
+      subject.finalize!
+      assert_invalid
+    end
+
+    it "keeps nil guest path if not provided" do
+      subject.synced_folder(".", name: "my-vagrant-folder")
+      sf = subject.synced_folders
+      expect(sf["my-vagrant-folder"][:guestpath]).to be_nil
+    end
+
+    context "WSL host paths" do
+      let(:valid_path){ "/mnt/c/path" }
+      let(:invalid_path){ "/home/vagrant/path" }
+      let(:synced_folder_impl){ double("synced_folder_impl", new: double("synced_folder_inst", usable?: true, _initialize: true)) }
+      let(:fs_config){ double("fs_config", vm: double("fs_vm", allowed_synced_folder_types: nil)) }
+      let(:plugin){ double("plugin", manager: manager) }
+      let(:manager){ double("manager", synced_folders: {sf_impl: [synced_folder_impl, 1]}) }
+
+      let(:stub_pathname){ double("stub_pathname", directory?: true, relative?: false) }
+
+      before do
+        allow(Pathname).to receive(:new).and_return(stub_pathname)
+        allow(stub_pathname).to receive(:expand_path).and_return(stub_pathname)
+        allow(Vagrant::Util::Platform).to receive(:wsl?).and_return(true)
+        allow(Vagrant::Util::Platform).to receive(:wsl_drvfs_path?).with(valid_path).and_return(true)
+        allow(Vagrant::Util::Platform).to receive(:wsl_drvfs_path?).with(invalid_path).and_return(false)
+        allow(machine).to receive(:config).and_return(fs_config)
+        allow(Vagrant).to receive(:plugin).with("2").and_return(plugin)
+        subject.synced_folder(".", "/vagrant", disabled: true)
+      end
+
+      it "is valid when located on DrvFs" do
+        subject.synced_folder(valid_path, "/guest/path")
+        subject.finalize!
+        assert_valid
+      end
+
+      it "is invalid when not located on DrvFs" do
+        subject.synced_folder(invalid_path, "/guest/path")
+        subject.finalize!
+        assert_invalid
+      end
+
+      context "when synced folder defines support for non-DrvFs" do
+        let(:support_nondrvfs){ true }
+
+        before do
+          allow(synced_folder_impl).to receive(:respond_to?).with(:wsl_allow_non_drvfs?).and_return(true)
+          allow(synced_folder_impl).to receive(:wsl_allow_non_drvfs?).and_return(support_nondrvfs)
+        end
+
+        context "and is supported" do
+          it "is valid when located on DrvFs" do
+            subject.synced_folder(valid_path, "/guest/path")
+            subject.finalize!
+            assert_valid
+          end
+
+          it "is valid when not located on DrvFs" do
+            subject.synced_folder(invalid_path, "/guest/path")
+            subject.finalize!
+            assert_valid
+          end
+        end
+
+        context "and is not supported" do
+          let(:support_nondrvfs){ false }
+
+          it "is valid when located on DrvFs" do
+            subject.synced_folder(valid_path, "/guest/path")
+            subject.finalize!
+            assert_valid
+          end
+
+          it "is invalid when not located on DrvFs" do
+            subject.synced_folder(invalid_path, "/guest/path")
+            subject.finalize!
+            assert_invalid
+          end
+        end
+      end
+    end
   end
 
   describe "#usable_port_range" do
