@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require_relative "../base"
 
 require "vagrant/util/platform"
@@ -15,7 +18,7 @@ describe VagrantPlugins::ProviderVirtualBox::Action::PrepareNFSSettings do
 
   let(:machine) do
     iso_env.machine(iso_env.machine_names[0], :dummy).tap do |m|
-      m.provider.stub(driver: driver)
+      allow(m.provider).to receive(:driver).and_return(driver)
     end
   end
 
@@ -29,12 +32,15 @@ describe VagrantPlugins::ProviderVirtualBox::Action::PrepareNFSSettings do
   before do
     env[:test] = true
     allow(machine.env).to receive(:host) { host }
+    allow(host).to receive(:capability?).with(:nfs_installed) { true }
     allow(host).to receive(:capability).with(:nfs_installed) { true }
+    # We don't care about smb support so return not installed
+    allow(host).to receive(:capability?).with(:smb_installed).and_return(false)
   end
 
   it "calls the next action in the chain" do
-    driver.stub(read_network_interfaces: {2 => {type: :hostonly, hostonly: "vmnet2"}})
-    driver.stub(read_host_only_interfaces: [{name: "vmnet2", ip: "1.2.3.4"}])
+    allow(driver).to receive(:read_network_interfaces).and_return({2 => {type: :hostonly, hostonly: "vmnet2"}})
+    allow(driver).to receive(:read_host_only_interfaces).and_return([{name: "vmnet2", ip: "1.2.3.4"}])
     allow(driver).to receive(:read_guest_ip).with(1).and_return("2.3.4.5")
 
     called = false
@@ -47,20 +53,22 @@ describe VagrantPlugins::ProviderVirtualBox::Action::PrepareNFSSettings do
   end
 
   context "with an nfs synced folder" do
+    let(:host_only_interfaces) {
+      [{name: "vmnet2", ip: "1.2.3.4"}]
+    }
+
     before do
       # We can't be on Windows, because NFS gets disabled on Windows
-      Vagrant::Util::Platform.stub(windows?: false)
+      allow(Vagrant::Util::Platform).to receive(:windows?).and_return(false)
 
       env[:machine].config.vm.synced_folder("/host/path", "/guest/path", type: "nfs")
       env[:machine].config.finalize!
 
       # Stub out the stuff so it just works by default
-      driver.stub(read_network_interfaces: {
+      allow(driver).to receive(:read_network_interfaces).and_return({
         2 => {type: :hostonly, hostonly: "vmnet2"},
       })
-      driver.stub(read_host_only_interfaces: [
-        {name: "vmnet2", ip: "1.2.3.4"},
-      ])
+      allow(driver).to receive(:read_host_only_interfaces).and_return(host_only_interfaces)
       allow(driver).to receive(:read_guest_ip).with(1).and_return("2.3.4.5")
 
       # override sleep to 0 so test does not take seconds
@@ -68,6 +76,30 @@ describe VagrantPlugins::ProviderVirtualBox::Action::PrepareNFSSettings do
       allow(subject).to receive(:retry_options).and_return(retry_options.merge(sleep: 0))
     end
 
+    context "with host interface netmask defined" do
+      context "with machine IP included within host interface range" do
+        let(:host_only_interfaces) {
+          [{name: "vmnet2", ip: "2.3.4.1", netmask: "255.255.255.0"}]
+        }
+
+        it "sets nfs_host_ip and nfs_machine_ip properly" do
+          subject.call(env)
+
+          expect(env[:nfs_host_ip]).to eq("2.3.4.1")
+          expect(env[:nfs_machine_ip]).to eq("2.3.4.5")
+        end
+      end
+      context "with machine IP included within host interface range" do
+        let(:host_only_interfaces) {
+          [{name: "vmnet2", ip: "1.2.3.4", netmask: "255.255.255.0"}]
+        }
+
+        it "raises an error when the machine IP is not within host interface range" do
+          expect{ subject.call(env) }.to raise_error(Vagrant::Errors::NFSNoHostonlyNetwork)
+        end
+      end
+    end
+
     it "sets nfs_host_ip and nfs_machine_ip properly" do
       subject.call(env)
 
