@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require_relative "../../../base"
 require "fake_ftp"
 
@@ -77,7 +80,7 @@ describe VagrantPlugins::FTPPush::FTPAdapter do
         ftp.upload("#{@dir}/file", "/file")
       end
 
-      expect(server.files).to include("file")
+      expect(server.files).to include("/file")
     end
 
     it "uploads in passive mode" do
@@ -86,7 +89,7 @@ describe VagrantPlugins::FTPPush::FTPAdapter do
         ftp.upload("#{@dir}/file", "/file")
       end
 
-      expect(server.file("file")).to be_passive
+      expect(server.file("/file")).to be_passive
     end
   end
 end
@@ -109,6 +112,7 @@ describe VagrantPlugins::FTPPush::SFTPAdapter do
   describe "#upload" do
     it "uploads the file" do
       pending "a way to mock an SFTP server"
+      test_with_mock_sftp_server
     end
   end
 end
