@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require File.expand_path("../../../../base", __FILE__)
 
 require Vagrant.source_root.join("plugins/synced_folders/nfs/config")
@@ -10,7 +13,7 @@ describe VagrantPlugins::SyncedFolderNFS::Config do
       subject.finalize!
     end
 
-    its(:functional) { should be_true }
+    its(:functional) { should be(true) }
     its(:map_gid) { should eq(:auto) }
     its(:map_uid) { should eq(:auto) }
   end
