@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "tempfile"
 require "tmpdir"
 
@@ -79,7 +82,7 @@ shared_context "unit" do
   # @return [Pathname]
   def temporary_dir
     # Create a temporary directory and append it to the instance
-    # variabe so that it isn't garbage collected and deleted
+    # variable so that it isn't garbage collected and deleted
     d = Dir.mktmpdir("vagrant-temporary-dir")
     @_temp_files ||= []
     @_temp_files << d
@@ -103,9 +106,10 @@ shared_context "unit" do
     allow(ENV).to receive(:[]).and_call_original
 
     hash.each do |key, value|
+      v = value.nil? ? nil : value.to_s
       allow(ENV).to receive(:[])
         .with(key.to_s)
-        .and_return(value.to_s)
+        .and_return(v)
     end
   end
 
