@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require File.expand_path("../../../../base", __FILE__)
 
 describe Vagrant::Action::Builtin::HandleBox do
@@ -37,6 +40,15 @@ describe Vagrant::Action::Builtin::HandleBox do
     subject.call(env)
   end
 
+  it "works if box is empty string" do
+    machine.config.vm.box = ""
+    machine.config.vm.box_url = nil
+
+    expect(app).to receive(:call).with(env)
+
+    subject.call(env)
+  end
+
   it "doesn't do anything if a box exists" do
     allow(machine).to receive(:box).and_return(box)
 
