@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require File.expand_path("../../../../base", __FILE__)
 
 require "vagrant/config/v2/util"
@@ -15,7 +18,7 @@ describe Vagrant::Config::V2::Util do
       }
 
       result = described_class.merge_errors(first, second)
-      result.should == expected
+      expect(result).to eq(expected)
     end
   end
 end
