@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require File.expand_path("../../base", __FILE__)
 require "json"
 require "pathname"
@@ -165,7 +168,7 @@ describe Vagrant::Environment do
 
         collection = double("collection")
         expect(Vagrant::BoxCollection).to receive(:new).with(
-          env.homedir.join("boxes"), anything).and_return(collection)
+          env.homedir.join("boxes"), anything).twice.and_return(collection)
         expect(collection).to receive(:upgrade_v1_1_v1_5).once
         subject
       end
@@ -641,6 +644,12 @@ VF
     it "uses a directory within the home directory by default" do
       klass = double("machine_index")
       stub_const("Vagrant::MachineIndex", klass)
+      # Need to stub the mappers here since the Vagrant::MachineIndex module is getting
+      # stubbed as a double. This is causing the mapper definition to fail on account
+      # of the Input to the mapper not being a module. The mapper has no impact on this 
+      # test otherwise.
+      stub_mapper = Class.new {}
+      stub_const("VagrantPlugins::CommandServe::Mappers", stub_mapper)
 
       expect(klass).to receive(:new).with(any_args) do |path|
         expect(path.to_s.start_with?(subject.home_path.to_s)).to be(true)
@@ -761,6 +770,7 @@ VF
     before do
       m = Vagrant.plugin("2").manager
       allow(m).to receive(:providers).and_return(plugin_providers)
+      allow_any_instance_of(described_class).to receive(:process_configured_plugins)
     end
 
     it "is the highest matching usable provider" do
@@ -986,7 +996,6 @@ VF
         Dir.chdir(temp_dir) do
           instance = described_class.new(local_data_path: "foo")
           expect(instance.local_data_path).to eq(instance.cwd.join("foo"))
-          expect(File.exist?(instance.local_data_path)).to be(false)
         end
       end
     end
@@ -1107,7 +1116,6 @@ VF
         end
 
         expect { instance }.to_not raise_error
-        expect(Pathname.new(local_data_path)).to_not be_exist
       end
 
       it "should upgrade all active VMs" do
@@ -1430,4 +1438,214 @@ VF
       expect(env.machine_names).to eq([:foo, :bar])
     end
   end
+
+  describe "guess_provider" do
+    before { allow_any_instance_of(described_class).to receive(:process_configured_plugins) }
+
+    it "should return the default provider by default" do
+      expect(subject).to receive(:default_provider).and_return("default_provider")
+      expect(subject.send(:guess_provider)).to eq("default_provider")
+    end
+
+    context "when provider is defined via command line argument" do
+      before { stub_const("ARGV", argv) }
+
+      context "when provider is given as single argument" do
+        let(:argv) { ["--provider=single_arg"] }
+
+        it "should return the provider name" do
+          expect(subject.send(:guess_provider)).to eq(:single_arg)
+        end
+      end
+
+      context "when provider is given as two arguments" do
+        let(:argv) { ["--provider", "double_arg"] }
+
+        it "should return the provider name" do
+          expect(subject.send(:guess_provider)).to eq(:double_arg)
+        end
+      end
+    end
+
+    context "when no default provider is available" do
+      before {
+        expect(subject).to receive(:default_provider).
+          and_raise(Vagrant::Errors::NoDefaultProvider) }
+
+      it "should return a nil value" do
+        expect(subject.send(:guess_provider)).to be_nil
+      end
+    end
+  end
+
+  describe "#find_configured_plugins" do
+    before do
+      allow_any_instance_of(described_class).to receive(:guess_provider).and_return(:dummy)
+      allow_any_instance_of(described_class).to receive(:process_configured_plugins)
+    end
+
+    it "should find no plugins when no plugins are configured" do
+      expect(subject.send(:find_configured_plugins)).to be_empty
+    end
+
+    context "when plugins are defined in the Vagrantfile" do
+      before do
+        env.vagrantfile <<-VF
+          Vagrant.configure("2") do |config|
+            config.vagrant.plugins = "vagrant-plugin"
+          end
+          VF
+      end
+
+      it "should return the vagrant-plugin" do
+        expect(subject.send(:find_configured_plugins).keys).to include("vagrant-plugin")
+      end
+    end
+
+    context "when plugins are defined in the Vagrantfile of a box" do
+      before do
+        env.box3("foo", "1.0", :dummy, vagrantfile: <<-VF)
+          Vagrant.configure("2") do |config|
+            config.vagrant.plugins = "vagrant-plugin"
+          end
+        VF
+        env.vagrantfile <<-VF
+          Vagrant.configure("2") do |config|
+            config.vm.box = "foo"
+          end
+        VF
+      end
+
+      it "should return the vagrant-plugin" do
+        expect(subject.send(:find_configured_plugins).keys).to include("vagrant-plugin")
+      end
+    end
+
+    context "when the box does not match the provider" do
+      before do
+        env.box3("foo", "1.0", :other, vagrantfile: <<-VF)
+          Vagrant.configure("2") do |config|
+            config.vagrant.plugins = "vagrant-plugin"
+          end
+        VF
+        env.vagrantfile <<-VF
+          Vagrant.configure("2") do |config|
+            config.vm.box = "foo"
+          end
+        VF
+      end
+
+      it "should not return the vagrant-plugin" do
+        expect(subject.send(:find_configured_plugins).keys).not_to include("vagrant-plugin")
+      end
+    end
+  end
+
+  describe "#process_configured_plugins" do
+    let(:env) do
+      isolated_environment.tap do |e|
+        e.box3("base", "1.0", :virtualbox)
+        e.vagrantfile(vagrantfile)
+      end
+    end
+
+    let(:vagrantfile) do
+      'Vagrant.configure("2"){ |config| config.vm.box = "base" }'
+    end
+
+    let(:plugin_manager) {
+      double("plugin_manager", installed_plugins: installed_plugins, local_file: local_file)
+    }
+
+    let(:installed_plugins) { {} }
+    let(:local_file) { double("local_file", installed_plugins: local_installed_plugins) }
+    let(:local_installed_plugins) { {} }
+
+    before do
+      allow(Vagrant::Plugin::Manager).to receive(:instance).and_return(plugin_manager)
+      allow(plugin_manager).to receive(:globalize!)
+      allow(plugin_manager).to receive(:localize!)
+      allow(plugin_manager).to receive(:load_plugins)
+    end
+
+    context "when local data directory does not exist" do
+      let(:local_file) { nil }
+
+      it "should properly return empty result" do
+        expect(instance.send(:process_configured_plugins)).to be_empty
+      end
+    end
+
+    context "plugins are disabled" do
+      before{ allow(Vagrant).to receive(:plugins_enabled?).and_return(false) }
+
+      it "should return empty result" do
+        expect(instance.send(:process_configured_plugins)).to be_nil
+      end
+    end
+
+    context "when vagrant is invalid" do
+      let(:vagrantfile) { 'Vagrant.configure("2"){ |config| config.vagrant.bad_key = true }' }
+
+      it "should raise a configuration error" do
+        expect { instance.send(:process_configured_plugins) }.to raise_error(Vagrant::Errors::ConfigInvalid)
+      end
+    end
+
+    context "with local plugins defined" do
+      let(:vagrantfile) { 'Vagrant.configure("2"){ |config| config.vagrant.plugins = "vagrant" }' }
+      let(:installed_plugins) { {"vagrant" => true} }
+
+      context "with plugin already installed" do
+
+        it "should not attempt to install a plugin" do
+          expect(plugin_manager).not_to receive(:install_plugin)
+          expect(instance.send(:process_configured_plugins)).to eq(local_installed_plugins)
+        end
+      end
+
+      context "without plugin installed" do
+
+        before { allow(instance).to receive(:exit) }
+
+        it "should prompt user before installation" do
+          expect(instance.ui).to receive(:ask).and_return("n")
+          expect(plugin_manager).to receive(:installed_plugins).and_return({})
+          expect { instance.send(:process_configured_plugins) }.to raise_error(Vagrant::Errors::PluginMissingLocalError)
+        end
+
+        it "should install plugin" do
+          expect(instance.ui).to receive(:ask).and_return("y")
+          expect(plugin_manager).to receive(:installed_plugins).and_return({})
+          expect(plugin_manager).to receive(:install_plugin).and_return(double("spec", "name" => "vagrant", "version" => "1"))
+          instance.send(:process_configured_plugins)
+        end
+
+        it "should exit after install" do
+          expect(instance.ui).to receive(:ask).and_return("y")
+          expect(plugin_manager).to receive(:installed_plugins).and_return({})
+          expect(plugin_manager).to receive(:install_plugin).and_return(double("spec", "name" => "vagrant", "version" => "1"))
+          expect(instance).to receive(:exit)
+          instance.send(:process_configured_plugins)
+        end
+      end
+    end
+  end
+
+  describe "#setup_local_data_path" do
+    before do
+      allow(FileUtils).to receive(:mkdir_p).and_call_original
+      allow(FileUtils).to receive(:cp).and_call_original
+    end
+
+    it "should create an rgloader path" do
+      expect(FileUtils).to receive(:mkdir_p).with(/(?!home)rgloader/)
+      instance
+    end
+
+    it "should write the rgloader file" do
+      expect(FileUtils).to receive(:cp).with(anything, /(?!home)rgloader.*rb$/)
+      instance
+    end
+  end
 end
