@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "json"
 require "pathname"
 require "tempfile"
@@ -105,6 +108,12 @@ describe Vagrant::MachineIndex do
 
     it "returns nil if the machine doesn't exist" do
       expect(subject.get("foo")).to be_nil
+      expect(subject.get(nil)).to be_nil
+    end
+
+    it "returns nil if the machine doesn't exist (is an empty string)" do
+      expect(subject.get("")).to be_nil
+      expect(subject.get(nil)).to be_nil
     end
 
     it "returns a valid entry if the machine exists" do
@@ -136,7 +145,11 @@ describe Vagrant::MachineIndex do
     end
 
     it "should include? by prefix" do
-      expect(subject.include?("b")).to be_true
+      expect(subject.include?("b")).to be(true)
+    end
+
+    it "should return false if given nil input" do
+      expect(subject.include?(nil)).to be(false)
     end
 
     it "locks the entry so subsequent gets fail" do
@@ -159,7 +172,7 @@ describe Vagrant::MachineIndex do
 
   describe "#include" do
     it "should not include non-existent things" do
-      expect(subject.include?("foo")).to be_false
+      expect(subject.include?("foo")).to be(false)
     end
 
     it "should include created entries" do
@@ -168,7 +181,7 @@ describe Vagrant::MachineIndex do
       subject.release(result)
 
       subject = described_class.new(data_dir)
-      expect(subject.include?(result.id)).to be_true
+      expect(subject.include?(result.id)).to be(true)
     end
   end
 
@@ -204,7 +217,7 @@ describe Vagrant::MachineIndex do
 
     it "can delete an entry that doesn't exist" do
       e = entry_klass.new
-      expect(subject.delete(e)).to be_true
+      expect(subject.delete(e)).to be(true)
     end
 
     it "updates an existing entry" do
@@ -268,6 +281,25 @@ describe Vagrant::MachineIndex do
       expect(entry.state).to eq(entry2.state)
     end
   end
+
+  describe "#recover" do
+    it "recovers an entry if not in the index" do
+      result = subject.recover(new_entry)
+      expect(result.id).to_not be_empty
+      expect { subject.get(result.id) }.to raise_error(Vagrant::Errors::MachineLocked)
+    end
+
+    it "returns an entry if in the index" do
+      test_entry = entry_klass.new()
+      entry = subject.set(test_entry)
+      subject.release(entry)
+
+      new_test_entry = entry_klass.new(id=entry.id, {})
+      result = subject.recover(new_test_entry)
+
+      expect(result.id).to eq(entry.id)
+    end
+  end
 end
 
 describe Vagrant::MachineIndex::Entry do
