@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require "pathname"
 
 require File.expand_path("../../base", __FILE__)
@@ -11,16 +14,16 @@ describe Vagrant::MachineState do
 
   it "should give access to the id" do
     instance = described_class.new(id, short, long)
-    instance.id.should == id
+    expect(instance.id).to eq(id)
   end
 
   it "should give access to the short description" do
     instance = described_class.new(id, short, long)
-    instance.short_description.should == short
+    expect(instance.short_description).to eq(short)
   end
 
   it "should give access to the long description" do
     instance = described_class.new(id, short, long)
-    instance.long_description.should == long
+    expect(instance.long_description).to eq(long)
   end
 end
