@@ -1,8 +1,12 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require File.expand_path("../../../../base", __FILE__)
 
 describe Vagrant::Plugin::V2::Plugin do
   before do
-    described_class.stub(manager: Vagrant::Plugin::V2::Manager.new)
+    allow(described_class).to receive(:manager)
+      .and_return(Vagrant::Plugin::V2::Manager.new)
   end
 
   it "should be able to set and get the name" do
@@ -71,8 +75,8 @@ describe Vagrant::Plugin::V2::Plugin do
         command("bar", primary: false) { "bar" }
       end
 
-      expect(plugin.components.commands[:foo][1][:primary]).to be_true
-      expect(plugin.components.commands[:bar][1][:primary]).to be_false
+      expect(plugin.components.commands[:foo][1][:primary]).to be(true)
+      expect(plugin.components.commands[:bar][1][:primary]).to be(false)
     end
 
     ["spaces bad", "sym^bols"].each do |bad|
