@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require File.expand_path("../../../../base", __FILE__)
 
 describe Vagrant::Plugin::V2::Provider do
@@ -42,14 +45,14 @@ describe Vagrant::Plugin::V2::Provider do
         end
       end
 
-      machine.stub(id: "YEAH")
+      allow(machine).to receive(:id).and_return("YEAH")
 
       instance._initialize("foo", machine)
     end
 
     it "can execute capabilities" do
-      expect(subject.capability?(:foo)).to be_false
-      expect(subject.capability?(:bar)).to be_true
+      expect(subject.capability?(:foo)).to be(false)
+      expect(subject.capability?(:bar)).to be(true)
 
       expect { subject.capability(:bar) }.
         to raise_error("bar YEAH")
