@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require File.expand_path("../../base", __FILE__)
 
 require "vagrant/shared_helpers"
@@ -8,14 +11,14 @@ describe Vagrant do
 
   subject { described_class }
 
-  describe "#global_lock" do
+  describe ".global_lock" do
     it "yields to the block" do
       result = subject.global_lock { 42 }
       expect(result).to eq(42)
     end
   end
 
-  describe "#in_installer?" do
+  describe ".in_installer?" do
     it "is not if env is not set" do
       with_temp_env("VAGRANT_INSTALLER_ENV" => nil) do
         expect(subject.in_installer?).to be(false)
@@ -29,7 +32,7 @@ describe Vagrant do
     end
   end
 
-  describe "#installer_embedded_dir" do
+  describe ".installer_embedded_dir" do
     it "returns nil if not in an installer" do
       allow(Vagrant).to receive(:in_installer?).and_return(false)
       expect(subject.installer_embedded_dir).to be_nil
@@ -44,7 +47,7 @@ describe Vagrant do
     end
   end
 
-  describe "#plugins_enabled?" do
+  describe ".plugins_enabled?" do
     it "returns true if the env is not set" do
       with_temp_env("VAGRANT_NO_PLUGINS" => nil) do
         expect(subject.plugins_enabled?).to be(true)
@@ -58,7 +61,7 @@ describe Vagrant do
     end
   end
 
-  describe "#server_url" do
+  describe ".server_url" do
     it "defaults to the default value" do
       with_temp_env("VAGRANT_SERVER_URL" => nil) do
         expect(subject.server_url).to eq(
@@ -92,7 +95,7 @@ describe Vagrant do
     end
   end
 
-  describe "#user_data_path" do
+  describe ".user_data_path" do
     around do |example|
       env = {
         "USERPROFILE" => nil,
@@ -119,7 +122,7 @@ describe Vagrant do
       end
     end
 
-    it "prefers VAGRANT_HOME over USERPOFILE if both are set" do
+    it "prefers VAGRANT_HOME over USERPROFILE if both are set" do
       env = {
         "USERPROFILE" => "/bar",
         "VAGRANT_HOME" => "/foo",
@@ -132,7 +135,7 @@ describe Vagrant do
     end
   end
 
-  describe "#prerelease?" do
+  describe ".prerelease?" do
     it "should return true when Vagrant version is development" do
       stub_const("Vagrant::VERSION", "1.0.0.dev")
       expect(subject.prerelease?).to be(true)
@@ -143,4 +146,77 @@ describe Vagrant do
       expect(subject.prerelease?).to be(false)
     end
   end
+
+  describe ".allow_prerelease_dependencies?" do
+    context "with environment variable set" do
+      before { allow(ENV).to receive(:[]).with("VAGRANT_ALLOW_PRERELEASE").and_return("1") }
+
+      it "should return true" do
+        expect(subject.allow_prerelease_dependencies?).to be(true)
+      end
+    end
+
+    context "with environment variable unset" do
+      before { allow(ENV).to receive(:[]).with("VAGRANT_ALLOW_PRERELEASE").and_return(nil) }
+
+      it "should return false" do
+        expect(subject.allow_prerelease_dependencies?).to be(false)
+      end
+    end
+  end
+
+  describe ".enable_resolv_replace" do
+    it "should not attempt to require resolv-replace by default" do
+      expect(subject).not_to receive(:require).with("resolv-replace")
+      subject.enable_resolv_replace
+    end
+
+    it "should require resolv-replace when VAGRANT_ENABLE_RESOLV_REPLACE is set" do
+      expect(subject).to receive(:require).with("resolv-replace")
+      with_temp_env("VAGRANT_ENABLE_RESOLV_REPLACE" => "1"){ subject.enable_resolv_replace }
+    end
+
+    it "should not require resolv-replace when VAGRANT_DISABLE_RESOLV_REPLACE is set" do
+      expect(subject).not_to receive(:require).with("resolv-replace")
+      with_temp_env("VAGRANT_ENABLE_RESOLV_REPLACE" => "1", "VAGRANT_DISABLE_RESOLV_REPLACE" => "1") do
+        subject.enable_resolv_replace
+      end
+    end
+  end
+
+  describe ".global_logger" do
+    after{ subject.global_logger = nil }
+
+    it "should return a logger when none have been provided" do
+      expect(subject.global_logger).not_to be_nil
+    end
+
+    it "should return previously set logger" do
+      logger = double("logger")
+      expect(subject.global_logger = logger).to eq(logger)
+      expect(subject.global_logger).to eq(logger)
+    end
+  end
+
+  describe ".add_default_cli_options" do
+    it "should raise a type error when no provided with proc" do
+      expect { subject.add_default_cli_options(true) }.
+        to raise_error(TypeError)
+    end
+
+    it "should raise an argument error when proc does not accept argument" do
+      expect { subject.add_default_cli_options(proc{}) }.
+        to raise_error(ArgumentError)
+    end
+
+    it "should accept a proc type argument" do
+      expect(subject.add_default_cli_options(proc{|o|})).to be_nil
+    end
+  end
+
+  describe ".default_cli_options" do
+    it "should return array of items" do
+      expect(subject.default_cli_options).to be_a(Array)
+    end
+  end
 end
