@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require File.expand_path("../../base", __FILE__)
 
 describe Vagrant::UI::Basic do
@@ -14,7 +17,7 @@ describe Vagrant::UI::Basic do
     end
 
     it "outputs using `puts` by default" do
-      expect(subject).to receive(:safe_puts).with(any_args) { |message, **opts|
+      expect(subject).to receive(:safe_puts).with(any_args) { |message, opts|
         expect(opts[:printer]).to eq(:puts)
         true
       }
@@ -23,7 +26,7 @@ describe Vagrant::UI::Basic do
     end
 
     it "outputs using `print` if new_line is false" do
-      expect(subject).to receive(:safe_puts).with(any_args) { |message, **opts|
+      expect(subject).to receive(:safe_puts).with(any_args) { |message, opts|
         expect(opts[:printer]).to eq(:print)
         true
       }
@@ -32,7 +35,7 @@ describe Vagrant::UI::Basic do
     end
 
     it "outputs using `print` if new_line is false" do
-      expect(subject).to receive(:safe_puts).with(any_args) { |message, **opts|
+      expect(subject).to receive(:safe_puts).with(any_args) { |message, opts|
         expect(opts[:printer]).to eq(:print)
         true
       }
@@ -44,7 +47,7 @@ describe Vagrant::UI::Basic do
       stdout = StringIO.new
       subject.stdout = stdout
 
-      expect(subject).to receive(:safe_puts).with(any_args) { |message, **opts|
+      expect(subject).to receive(:safe_puts).with(any_args) { |message, opts|
         expect(opts[:io]).to be(stdout)
         true
       }
@@ -60,7 +63,7 @@ describe Vagrant::UI::Basic do
       stderr = StringIO.new
       subject.stderr = stderr
 
-      expect(subject).to receive(:safe_puts).with(any_args) { |message, **opts|
+      expect(subject).to receive(:safe_puts).with(any_args) { |message, opts|
         expect(opts[:io]).to be(stderr)
         true
       }
@@ -81,7 +84,7 @@ describe Vagrant::UI::Basic do
 
   context "#detail" do
     it "outputs details" do
-      expect(subject).to receive(:safe_puts).with(any_args) { |message, **opts|
+      expect(subject).to receive(:safe_puts).with(any_args) { |message, opts|
         expect(message).to eq("foo")
         true
       }
@@ -104,12 +107,39 @@ describe Vagrant::UI::Basic do
     before{ Vagrant::Util::CredentialScrubber.sensitive(password) }
 
     it "should remove sensitive information from the output" do
-      expect(subject).to receive(:safe_puts).with(any_args) do |message, **opts|
+      expect(subject).to receive(:safe_puts).with(any_args) do |message, opts|
         expect(message).not_to include(password)
       end
       subject.detail(output)
     end
   end
+
+  context "#rewriting" do
+    it "does output progress" do
+      expect { |b| subject.rewriting(&b) }.to yield_control
+    end
+  end
+end
+
+describe Vagrant::UI::NonInteractive do
+  describe "#ask" do
+    it "raises an exception" do
+      expect{subject.ask("foo")}.to raise_error(Vagrant::Errors::UIExpectsTTY)
+    end
+  end
+
+  describe "#report_progress" do
+    it "does not output progress" do
+      expect(subject).to_not receive(:info)
+      subject.report_progress(1, 1)
+    end
+  end
+
+  describe "#rewriting" do
+    it "does not output progress" do
+      expect { |b| subject.rewriting(&b) }.to_not yield_control
+    end
+  end
 end
 
 describe Vagrant::UI::Colored do
@@ -352,12 +382,12 @@ describe Vagrant::UI::Prefixed do
 
   describe "#machine" do
     it "sets the target option" do
-      expect(ui).to receive(:machine).with(:foo, target: prefix)
+      expect(ui).to receive(:machine).with(:foo, { target: prefix })
       subject.machine(:foo)
     end
 
     it "preserves existing options" do
-      expect(ui).to receive(:machine).with(:foo, :bar, foo: :bar, target: prefix)
+      expect(ui).to receive(:machine).with(:foo, :bar, { foo: :bar, target: prefix })
       subject.machine(:foo, :bar, foo: :bar)
     end
   end
@@ -385,7 +415,7 @@ describe Vagrant::UI::Prefixed do
       subject.output("foo\nbar")
     end
 
-    it "doesn't prefix if requestsed" do
+    it "doesn't prefix if requested" do
       expect(ui).to receive(:output).with("foo", prefix: false, bold: true, target: prefix)
       subject.output("foo", prefix: false)
     end
@@ -406,4 +436,17 @@ describe Vagrant::UI::Prefixed do
       subject.output("foo", target: "bar")
     end
   end
+
+  describe "#format_message" do
+    it "should return the same number of new lines as given" do
+      ["no new line", "one\nnew line", "two\nnew lines\n", "three\nnew lines\n\n"].each do |msg|
+        expect(subject.format_message(:detail, msg).count("\n")).to eq(msg.count("\n"))
+      end
+    end
+
+    it "should properly format a blank message" do
+      expect(subject.format_message(:detail, "", target: "default", prefix: true)).
+        to match(/\s+default:\s+/)
+    end
+  end
 end
