@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require File.expand_path("../../../base", __FILE__)
 
 require "vagrant/util/ansi_escape_code_remover"
@@ -10,7 +13,7 @@ describe Vagrant::Util::ANSIEscapeCodeRemover do
   end
 
   it "should remove ANSI escape codes" do
-    klass.remove_ansi_escape_codes("\e[Hyo").should == "yo"
+    expect(klass.remove_ansi_escape_codes("\e[Hyo")).to eq("yo")
   end
 end
 
