@@ -1,3 +1,6 @@
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 require File.expand_path("../../../base", __FILE__)
 
 require "vagrant/util/line_ending_helpers"
@@ -10,7 +13,7 @@ describe Vagrant::Util::LineEndingHelpers do
   end
 
   it "should convert DOS to unix-style line endings" do
-    klass.dos_to_unix("foo\r\nbar\r\n").should == "foo\nbar\n"
+    expect(klass.dos_to_unix("foo\r\nbar\r\n")).to eq("foo\nbar\n")
   end
 end
 
